
#include "lex_analyse.h"
#include "semantic.h"

const Token t_extern = { tt_extern, "extern", 6 };
const Token t_static = { tt_static, "static", 6 };

const Token t_minus     = { tt_minus,     "-", 1 };
const Token t_semicolon = { tt_semicolon, ";", 1 };
const Token t_equal     = { tt_equal,     "=", 1 };
const Token t_white     = { tt_comment,   " ", 1 };

class Semantic
{
    Suite<TreeNode> & tree;
// 
    void operation ( nat32 i );
    void expression2 ( nat32 i );
    void expression1 ( nat32 i );
    void expressionBracesClose2 ( nat32 i );
    void expressionBracesClose1 ( nat32 i );
    void expressionBracesClose  ( nat32 i );
    void expressionParenthesisClose2 ( nat32 i );
    void expressionParenthesisClose1 ( nat32 i );
    void expressionParenthesisClose  ( nat32 i );
    void expressionSemicolon2 ( nat32 i );
    void expressionSemicolon1 ( nat32 i );
    void expressionSemicolon  ( nat32 i );
    void expressionList3 ( nat32 i );
    void expressionList2 ( nat32 i );
    void expressionList1 ( nat32 i );
    void expressionList  ( nat32 i );
// 
    void dataDefenition( nat32 i );
    void statement1    ( nat32 i );
    void statement     ( nat32 i );
    void statementList ( nat32 i );
// 
    void return_type    ( nat32 i );
    void argument       ( nat32 i );
    void argumentList2  ( nat32 i );
    void argumentList   ( nat32 i );
    void subProgramType ( nat32 i, nat32 iname );
    void definition     ( nat32 i, nat32 iname );
    void declaration    ( nat32 i );
public:
    Semantic ( Suite<TreeNode> & tr ) : tree ( tr ) {}
    void run ();
};

/***************************************************************/
//
//                        
//
/***************************************************************/

void Semantic::operation ( nat32 i )
{
}

void Semantic::expression2 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_parenthesis_open:
        expressionList ( tree[i].right );
    }
}

void Semantic::expression1 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_plus:
    case tt_minus:
    case tt_exclamation:
    case tt_plus_plus:
    case tt_minus_minus:
        expression1 ( tree[i].right );
        break;
    case tt_parenthesis_open:
        expressionParenthesisClose1 ( tree[i].right );
        break;
    }
}

void Semantic::expressionBracesClose2 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_Expression2:
        expression2 ( i );
        i = tree[i].right;
        expressionBracesClose2 ( i );
        break;
    case tt_Operation:
        operation ( i );
        i = tree[i].right;
        expressionBracesClose1 ( i );
    }
}

void Semantic::expressionBracesClose1 ( nat32 i )
{
    i = tree[i].down;
    expression1 ( i );
    i = tree[i].right;
    expressionBracesClose2 ( i );
}

void Semantic::expressionBracesClose ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_ExpressionBracesClose1 )
    {
        expressionBracesClose1 ( i );
    }
}

void Semantic::expressionParenthesisClose2 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_Expression2:
        expression2 ( i );
        i = tree[i].right;
        expressionParenthesisClose2 ( i );
        break;
    case tt_Operation:
        operation ( i );
        i = tree[i].right;
        expressionParenthesisClose1 ( i );
    }
}

void Semantic::expressionParenthesisClose1 ( nat32 i )
{
    i = tree[i].down;
    expression1 ( i );
    i = tree[i].right;
    expressionParenthesisClose2 ( i );
}

void Semantic::expressionParenthesisClose ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_ExpressionParenthesisClose1 )
    {
        expressionParenthesisClose1 ( i );
    }
}

void Semantic::expressionSemicolon2 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_Expression2:
        expression2 ( i );
        i = tree[i].right;
        expressionSemicolon2 ( i );
        break;
    case tt_Operation:
        operation ( i );
        i = tree[i].right;
        expressionSemicolon1 ( i );
    }
}

void Semantic::expressionSemicolon1 ( nat32 i )
{
    i = tree[i].down;
    expression1 ( i );
    i = tree[i].right;
    expressionSemicolon2 ( i );
}

void Semantic::expressionSemicolon ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_ExpressionSemicolon1 )
    {
        expressionSemicolon1 ( i );
    }
}

void Semantic::expressionList3  ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_comma )
    {
        i = tree[i].right;
        expressionList ( i );
    }
}

void Semantic::expressionList2  ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_ExpressionList3:
        expressionList3 ( i );
        break;
    case tt_Expression2:
        expression2 ( i );
        i = tree[i].right;
        expressionList2 ( i );
        break;
    case tt_Operation:
        operation ( i );
        i = tree[i].right;
        expressionList1 ( i );
    }
}

void Semantic::expressionList1  ( nat32 i )
{
    i = tree[i].down;
    expression1 ( i );
    i = tree[i].right;
    expressionList2 ( i );
}

void Semantic::expressionList  ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_ExpressionList1 )
    {
        expressionList1 ( i );
    }
}

/***************************************************************/
//
//                        
//
/***************************************************************/

void Semantic::dataDefenition ( nat32 i )
{
    const nat32 iname = tree[i].down;
    const nat32 idef2 = tree[iname].right;
    tree[idef2].left = 0;
    tree[tree[idef2].up].down = idef2;
    i = tree[idef2].down;
    if( tree[i].token.type != tt_DataType ) i = tree[i].right;
    TreeNode node;
    node.comment = & t_white;
    node.down = 0;
    node.token = t_equal;
    node.left = iname;
    node.right = tree[i].right;
    node.up = tree[iname].up = tree[i].up;
    tree[i].right = iname;
    tree[iname].left = i;
    tree[node.right].left = tree[iname].right = tree.addAftLas ( node );
    i = node.right;
    while ( tree[i].right != 0 ) i = tree[i].right;
    node.comment = 0;
    node.down = 0;
    node.token = t_semicolon;
    node.left = i;
    node.right = 0;
    node.up = tree[i].up;
    tree[i].right = tree.addAftLas ( node );
    expressionBracesClose ( i );
}

void Semantic::statement1 ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_if:
        i = tree[i].right;
        i = tree[i].right;
        expressionParenthesisClose1 ( i );
        i = tree[i].right;
        statement1 ( i );
        i = tree[i].right;
        i = tree[i].right;
        statement1 ( i );
        break;
    case tt_for:
        i = tree[i].right;
        i = tree[i].right;
        expressionSemicolon ( i );
        i = tree[i].right;
        expressionSemicolon ( i );
        i = tree[i].right;
        expressionParenthesisClose ( i );
        i = tree[i].right;
        statement1 ( i );
        break;
    case tt_break:
    case tt_continue:
        TreeNode node;
        node.down = 0;
        node.token = t_semicolon;
        node.left = i;
        node.right = 0;
        node.up = tree[i].up;
        node.comment = 0;
        tree[i].right = tree.addAftLas ( node );
        break;
    case tt_return:
        i = tree[i].right;
        expressionSemicolon ( i );
        break;
    case tt_braces_open:
        i = tree[i].right;
        statementList ( i );
        break;
    case tt_ExpressionSemicolon1:
        expressionSemicolon1 ( i );
    }
}

void Semantic::statement ( nat32 i )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_auto:
    case tt_static:
        dataDefenition ( tree[i].right );
        break;
    default:
        statement1 ( i );
    }
}

void Semantic::statementList ( nat32 i )
{
    while ( tree[i].token.type == tt_StatementList )
    {
        i = tree[i].down;
        if ( tree[i].token.type == tt_braces_close )
        {
            break;
        }
        statement ( i );
        i = tree[i].right;
    }
}

/***************************************************************/
//
//                        
//
/***************************************************************/

void Semantic::return_type ( nat32 i )
{
/*    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_void:
//        result += vtoken[node.index]; // void
        break;
    case tt_DataType:
//        result += vtoken[tree[tree[i].down].index]; // data type
        break;
    }*/
}

void Semantic::argument ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type == tt_name )
    {
        const nat32 iname = i;
        const nat32 iarg2 = tree[i].right;
        if ( tree[iname].comment == 0 )
        {
            TreeNode node;
            node.down = 0;
            node.up = tree[i].up;
            node.token = t_white;
            node.left = iarg2;
            node.right = iname;
            node.comment = 0;
            tree[iarg2].right = tree[iname].left = tree.addAftLas ( node );
        }
        else
        {
            tree[iarg2].right = iname;
            tree[iname].left = iarg2;
        }
        tree[tree[i].up].down = iarg2;
        tree[iname].right = tree[iarg2].left = 0;
    }
}

void Semantic::argumentList2 ( nat32 i )
{
    const TreeNode & node = tree[tree[i].down];
    if ( node.token.type != tt_parenthesis_close )
    {
        argumentList ( node.right );
    }
}

void Semantic::argumentList ( nat32 i )
{
    i = tree[i].down;
    if ( tree[i].token.type != tt_parenthesis_close )
    {
        argument ( i );
        argumentList2 ( tree[i].right );
    }
}

void Semantic::subProgramType ( nat32 i, nat32 iname )
{
    i = tree[i].down;
    TreeNode & node = tree[i];
    const nat32 iarg = tree[i].right; // ArgumentList
    argumentList ( iarg );
    i = tree[iarg].right; // ReturnType
    tree[iarg].right = 0;
    tree[iname].up = node.up;
    tree[i].left = 0;
    tree[i].right = iname;
    tree[iname].right = tree[node.up].down;
    tree[tree[iname].right].left = iname;
    tree[node.up].down = tree[iname].left = i;
}

void Semantic::definition ( nat32 i, nat32 iname )
{
    i = tree[i].down;
    switch ( tree[i].token.type )
    {
    case tt_const:
            i = tree[i].right;
    case tt_Definition2:
        {
            const nat32 idef2 = i;
            i = tree[i].down; // DataType
            tree[iname].right = tree[i].right;
            tree[tree[i].right].left = iname;
            tree[iname].left = i;
            tree[i].right = iname;
            tree[iname].up = tree[i].up;
            nat32 ib = tree[iname].right;
            nat32 ie = tree[ib].right;
            nat32 j = tree[ie].down;
            if ( tree[j].token.type == tt_braces_close )
            {
                tree[ib].token = t_semicolon;
                tree[ib].right = 0;
            }
            else
            {
                TreeNode node2;
                node2.down = 0;
                node2.token = t_semicolon;
                node2.left = idef2;
                node2.right = 0;
                node2.up = tree[idef2].up;
                node2.comment = 0;
                tree[idef2].right = tree.addAftLas ( node2 );
                node2.comment = & t_white;
                node2.down = 0;
                node2.token = t_equal;
                node2.left = iname;
                node2.right = tree[iname].right;
                node2.up = tree[iname].up;
                tree[iname].right = tree[node2.right].left = tree.addAftLas ( node2 );
            }
        }
        break;
    case tt_SubProgramType:
        {
            subProgramType ( i, iname );
            i = tree[i].right;
            i = tree[i].right;
            statementList ( i );
        }
        break;
    }
}

void Semantic::declaration ( nat32 i )
{
    i = tree[i].down;
    TreeNode & node = tree[i];
    switch ( node.token.type )
    {
    case tt_local:
        {
            node.token = t_static;
            const nat32 iname = node.right;
            const nat32 idef = tree[iname].right;
            node.right = idef;
            tree[idef].left = i;
            definition ( idef, iname );
        }
        break;
    case tt_global:
        {
            const nat32 iname = node.right;
            const nat32 idef = tree[iname].right;
            node.right = idef;
            tree[idef].left = i;
            definition ( idef, iname );
            if ( tree[tree[idef].down].token.type == tt_const )
            {
                node.token = t_extern;
            }
            else
            {
                const Token token = { tt_comment, "/****/", 6 };
                node.token = token;
            }
        }
        break;
    case tt_extern:
        {
            const nat32 iname = tree[i].right;
            const nat32 itype = tree[iname].right;
            nat32 j = tree[itype].down;
            TreeNode node2;
            node2.down = 0;
            node2.token = t_semicolon;
            node2.right = 0;
            node2.up = node.up;
            node2.comment = 0;
            if ( tree[j].token.type == tt_SubProgramType )
            {
                node.right = itype;
                tree[itype].left = i;
                subProgramType ( j, iname );
                node2.left = itype;
            }
            else
            {
                tree[i].right = itype;
                tree[itype].right = iname;
                tree[itype].left = i;
                tree[iname].left = itype;
                node2.left = iname;
            }
            tree[node2.left].right = tree.addAftLas ( node2 );
        }
        break;
    }
}

void Semantic::run ()
{
    nat32 i = 1;
    for(;;)
    {
        switch ( tree[i].token.type )
        {
        case tt_DeclarationList:
            i = tree[i].down;
            continue;
        case tt_Declaration:
            declaration ( i );
            i = tree[i].right;
            continue;
        }
        break;
    }
}

void semantic ( Suite<TreeNode> & tree )
{
    Semantic sem ( tree );
    sem.run();
}
