
#ifndef SHEVARRAY_H
#define SHEVARRAY_H

#include "template.h"

#undef NAT
#define NAT unsigned int

/********************** CArrRef ************************/
/************     *************/

template <class T> class CArrRef
{
    void operator = ( const CArrRef & );
protected:
    const T * _data;
    NAT _size;
public:
    CArrRef ( const T * d, NAT n ) : _data(d), _size(n) {}
    const T & operator[] ( NAT i ) const { return _data[i]; }
    NAT size () const { return _size; }
};

/*********************** ArrRef ************************/
/******************    *******************/

template <class T> class ArrRef : public CArrRef<T>
{
    void operator = ( const ArrRef & );
public:
    ArrRef ( T * d, NAT n ) : CArrRef<T>( d, n ) {}
    T & operator[] ( NAT i ) { return ((T*)_data)[i]; }
    const T & operator[] ( NAT i ) const { return _data[i]; }
};

/*********************** FixArray ************************/
/***************    *****************/

template <class T, NAT n> class FixArray : public ArrRef<T>
{
    T stor[n];
    FixArray ( const FixArray & );
public:
    FixArray () : ArrRef<T> ( stor, n ) {}
    FixArray & operator= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] = a._data[i];
        return *this;
    }
    FixArray & operator+= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] += a._data[i];
        return *this;
    }
    FixArray & operator-= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] -= a._data[i];
        return *this;
    }
    FixArray & operator*= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] *= a._data[i];
        return *this;
    }
    FixArray & operator/= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] /= a._data[i];
        return *this;
    }
    FixArray & operator%= ( const FixArray & a )
    {
        for ( NAT i = 0; i < n; ++i ) ((T*)_data)[i] %= a._data[i];
        return *this;
    }
};

/********************** DynArray ***********************/
/****************   ******************/

template <class T> class DynArray : public ArrRef<T>
{
    DynArray ( const DynArray & );
public:
explicit DynArray ( NAT n = 0 ) : ArrRef<T> ( new T[n], n ) {}
        ~DynArray () { delete[] (T*)_data; }

    DynArray & operator= ( const CArrRef<T> & r )
    {
        if ( _size != r.size() )
        {
            _size = r.size();
            delete[] (T*)_data;
            _data = new T[_size];
        }
        for ( NAT i = 0; i < _size; ++i ) ((T*)_data)[i] = r[i];
        return *this;
    }

    DynArray & operator= ( const DynArray & a )
    {
        return this->operator= ( ( const CArrRef<T> &) a );
    }

    void resize ( NAT n )
    {
        delete[] (T*)_data;
        _data = new T[_size=n];
    }

    void resizeAndCopy ( NAT n )
    {
        const NAT k = n > _size ? _size : n;
        T * tmp = new T[_size=n];
        for ( NAT i = 0; i < k; ++i ) tmp[i] = _data[i];
        delete[] (T*)_data;
        _data = tmp;
    }

    friend void swap ( DynArray & a1, DynArray & a2 );
};

template <class T> 
void swap ( DynArray<T> & a1, DynArray<T> & a2 )
{
    _swap ( a1._size, a2._size );
    _swap ( a1._data, a2._data );
}

/*********************** Suite *************************/
/**********    ***********/

template <class T> class Suite : public ArrRef<T>
{
    NAT real_size;

    void resizeAndCopy ( NAT n )
    {
        T * tmp = new T[n];
        for ( NAT i = 0; i < real_size; ++i ) tmp[i] = _data[i];
        delete[] (T*)_data;
        _data = tmp;
        real_size = n;
    }

    Suite ( const Suite & );
    void operator= ( const Suite & );

public:

    explicit Suite ( NAT n = 0 ) : ArrRef<T>(new T[real_size=n==0?32:n], n) {}
    ~Suite () { delete[] (T*)_data; }

    NAT addAftLas ( const T & x )
    {
        if ( _size == real_size )
        {
            resizeAndCopy ( _size + _size );
        }
        ((T*)_data)[_size] = x;
        return _size++;
    }

    void addAftLas ( const CArrRef<T> & a )
    {
        const NAT n = _size + a.size();
        NAT s = real_size;
        if ( s < n )
        {
            while ( s < n ) s += s;
            resizeAndCopy ( s );
        }
        for ( NAT i = 0; i < a.size(); ++i ) ((T*)_data)[_size+i] = a[i];
        _size = n;
    }

    void resize ( NAT n )
    {
        if ( n > real_size ) resizeAndCopy ( n );
        _size = n;
    }

    void dec ()
    {
        if ( _size > 0 ) --_size;
    }

    friend void swap ( Suite & a1, Suite & a2 );
};

template <class T> 
void swap ( Suite<T> & a1, Suite<T> & a2 )
{
    _swap ( a1.real_size, a2.real_size );
    _swap ( a1._size, a2._size );
    _swap ( a1._data, a2._data );
}

/****************    *****************/

template <class T> 
bool operator == ( const CArrRef<T> & a, const CArrRef<T> & b )
{
    if ( a.size() != b.size() ) return false;
    for ( NAT i = 0; i < a.size(); ++i ) 
        if ( a[i] != b[i] ) return false;
    return true;
}

template <class T> 
bool operator != ( const CArrRef<T> & a, const CArrRef<T> & b )
{
    if ( a.size() != b.size() ) return true;
    for ( NAT i = 0; i < a.size(); ++i ) 
        if ( a[i] != b[i] ) return true;
    return false;
}

template <class T> 
NAT count ( const CArrRef<T> & a, const T & b, NAT from = 0 )
{
    NAT n = 0;
    for ( NAT i = from; i < a.size(); ++i ) 
        if ( a[i] == b ) ++n;
    return n;
}

template <class T> 
NAT count ( const CArrRef<T> & a, const T & b, NAT from, NAT before )
{
    NAT n = 0;
    if ( before > a.size() ) before = a.size();
    for ( NAT i = from; i < before; ++i ) 
        if ( a[i] == b ) ++n;
    return n;
}

template <class T> 
NAT firEqu ( const CArrRef<T> & a, const T & b, NAT from = 0 )
{
    for ( NAT i = from; i < a.size(); ++i ) 
        if ( a[i] == b ) return i;
    return a.size();
}

template <class T> 
NAT firEqu ( const CArrRef<T> & a, const T & b, NAT from, NAT before )
{
    if ( before > a.size() ) before = a.size();
    for ( NAT i = from; i < before; ++i ) 
        if ( a[i] == b ) return i;
    return a.size();
}

template <class T> 
ArrRef<T> & squ ( ArrRef<T> & a, NAT from = 0 )
{
    for ( NAT i = from; i < a.size(); ++i ) a[i] *= a[i];
    return a;
}

template <class T> 
ArrRef<T> & squ ( ArrRef<T> & a, NAT from, NAT before )
{
    if ( before > a.size() ) before = a.size();
    for ( NAT i = from; i < before; ++i ) a[i] *= a[i];
    return a;
}

/*********************** For012 ************************/

template <class T> class For012
{
    ArrRef<T> & ref;
public:
    explicit For012 ( ArrRef<T> & a ) : ref ( a ) {}
    template <class T2> For012<T> & operator = ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] = a;
        return *this;
    }
    template <class T2> For012<T> & operator += ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] += a;
        return *this;
    }
    template <class T2> For012<T> & operator -= ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] -= a;
        return *this;
    }
    template <class T2> For012<T> & operator *= ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] *= a;
        return *this;
    }
    template <class T2> For012<T> & operator /= ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] /= a;
        return *this;
    }
    template <class T2> For012<T> & operator %= ( const T2 & a )
    {
        for ( NAT i = 0; i < ref.size(); ++i ) ref[i] %= a;
        return *this;
    }
};

template <class T> 
inline For012<T> for012 ( ArrRef<T> & a )
{
    return For012<T> ( a );
}

/*********************** CFor012 ************************/

template <class T> class CFor012
{
public:
    const CArrRef<T> & ref;
    explicit CFor012 ( const CArrRef<T> & a ) : ref ( a ) {}
};

template <class T> 
inline CFor012<T> cfor012 ( const CArrRef<T> & a )
{
    return CFor012<T> ( a );
}

template <class T1, class T2> 
T1 & operator += ( T1 & t, const CFor012<T2> & a )
{
    for ( NAT i = 0; i < a.ref.size(); ++i ) t += a.ref[i];
    return t;
}

#undef NAT

#endif