
#include "ShevList.h"

namespace SL
{

ShevItem::~ShevItem() {}

//***************************************************************************//

bool InfoEQ::operator () ( const ShevItem * p ) { return p->info == i; }
bool InfoNE::operator () ( const ShevItem * p ) { return p->info != i; }
bool InfoLT::operator () ( const ShevItem * p ) { return p->info <  i; }
bool InfoGT::operator () ( const ShevItem * p ) { return p->info >  i; }
bool InfoLE::operator () ( const ShevItem * p ) { return p->info <= i; }
bool InfoGE::operator () ( const ShevItem * p ) { return p->info >= i; }

bool NotPredicate1::operator () ( const ShevItem * p ) { return ! pre ( p ); }
bool Or_Predicate1::operator () ( const ShevItem * p ) { return pre1 ( p ) || pre2 ( p ); }
bool AndPredicate1::operator () ( const ShevItem * p ) { return pre1 ( p ) && pre2 ( p ); }

//***************************************************************************//

void Alter::operator () ( ShevItem * ) {}

Alter none;

void InfoAsn::operator () ( ShevItem * p ) { p->info  = i; }
void InfoAdd::operator () ( ShevItem * p ) { p->info += i; }
void InfoSub::operator () ( ShevItem * p ) { p->info -= i; }
void InfoMul::operator () ( ShevItem * p ) { p->info *= i; }
void InfoDiv::operator () ( ShevItem * p ) { p->info /= i; }

void If::operator () ( ShevItem * p ) { if ( pre(p) ) alt1(p); else alt2(p); }

//***************************************************************************//

InfoActGen Info;

//***************************************************************************//

bool Double123Sorter::operator () ( const ShevItem * item1, const ShevItem * item2 )
{
    return array[item1->info] <= array[item2->info];
}

bool Double321Sorter::operator () ( const ShevItem * item1, const ShevItem * item2 )
{
    return array[item1->info] >= array[item2->info];
}

//***************************************************************************//

ShevList::~ShevList()
{
    while ( ( cPtr = fPtr ) != 0 )
    {
        fPtr = fPtr->nextPtr;
        delete cPtr;
    }
}

void ShevList::delAll ()
{
    while ( ( cPtr = fPtr ) != 0 )
    {
        fPtr = fPtr->nextPtr;
        delete cPtr;
    }
    lPtr = 0;
    listSize = 0;
}

//     

unsigned int ShevList::count ( Predicate1 & pre ) const
{
    unsigned int n = 0;
    const ShevItem * p = fPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) ++n;
        p = p->nextPtr;
    }
    return n;
}

unsigned int ShowList::count ( Predicate1 & pre ) const
{
    unsigned int n = 0;
    const ShevItem * p = fPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) ++n;
        p = p->nextPtr;
    }
    return n;
}

//     

const ShevItem * ShevList::fir ( Predicate1 & pre ) const
{
    const ShevItem * p = fPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) break;
        p = p->nextPtr;
    }
    return p;
}

const ShevItem * ShowList::fir ( Predicate1 & pre ) const
{
    const ShevItem * p = fPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) break;
        p = p->nextPtr;
    }
    return p;
}

//     

const ShevItem * ShevList::las ( Predicate1 & pre ) const
{
    const ShevItem * p = lPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) break;
        p = p->prevPtr;
    }
    return p;
}

const ShevItem * ShowList::las ( Predicate1 & pre ) const
{
    const ShevItem * p = lPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) ) break;
        p = p->prevPtr;
    }
    return p;
}

//       

bool ShevList::findFir ( Predicate1 & pre )
{
    ShevItem * p = fPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) )
        {
            cPtr = p;
            return true;
        }
        p = p->nextPtr;
    }
    return false;
}

//       

bool ShevList::findLas ( Predicate1 & pre )
{
    ShevItem * p = lPtr;
    while ( p != 0 )
    {
        if ( pre ( p ) )
        {
            cPtr = p;
            return true;
        }
        p = p->prevPtr;
    }
    return false;
}

void ShevList::addBefFir ( ShevItem * m )
{
    if ( ! m ) return;
    if ( cPtr )
    {
        m->nextPtr = fPtr;
        fPtr = fPtr->prevPtr = m;
    }
    else
    {
        m->nextPtr = 0;
        cPtr = fPtr = lPtr = m;
    }
    m->prevPtr = 0;
    ++listSize;
}

void ShevList::addBefCur ( ShevItem * m )
{
    if ( ! m ) return;
    if ( cPtr )
    {
        m->prevPtr = cPtr->prevPtr;
        m->nextPtr = cPtr;
        cPtr->prevPtr = m;
        if ( m->prevPtr ) m->prevPtr->nextPtr = m;
        else fPtr = m;
    }
    else
    {
        m->prevPtr = m->nextPtr = 0; fPtr = lPtr = m;
    }
    cPtr = m;
    ++listSize;
}

void ShevList::addAftCur ( ShevItem * m )
{
    if ( ! m ) return;
    if ( cPtr )
    {
        m->prevPtr = cPtr;
        m->nextPtr = cPtr->nextPtr;
        cPtr->nextPtr = m;
        if ( m->nextPtr ) m->nextPtr->prevPtr = m;
        else lPtr = m;
    }
    else
    {
        m->prevPtr = m->nextPtr = 0; fPtr = lPtr = m;
    }
    cPtr = m;
    ++listSize;
}

void ShevList::addAftLas ( ShevItem * m )
{
    if ( ! m ) return;
    if ( cPtr )
    {
        m->prevPtr = lPtr;
        lPtr = lPtr->nextPtr = m;
    }
    else
    {
        m->prevPtr = 0;
        cPtr = fPtr = lPtr = m;
    }
    m->nextPtr = 0;
    ++listSize;
}

void ShevList::movCurBefFir ( ShevList & list )
{
    if ( !cPtr ) return;
    ShevItem * m = cPtr;
    if ( m->nextPtr ) ( cPtr = m->nextPtr )->prevPtr = m->prevPtr;
    else lPtr = cPtr = m->prevPtr;
    if ( m->prevPtr ) m->prevPtr->nextPtr = m->nextPtr;
    else fPtr = cPtr;
    --listSize;
    if ( list.lPtr )
    {
        m->nextPtr = list.fPtr;
        list.fPtr = list.fPtr->prevPtr = m;
    }
    else
    {
        m->nextPtr = 0;
        list.cPtr = list.fPtr = list.lPtr = m;
    }
    m->prevPtr = 0;
    ++list.listSize;
}

void ShevList::movCurBefCur ( ShevList & list )
{
    if ( !cPtr ) return;
    ShevItem * m = cPtr;
    if ( m->nextPtr ) ( cPtr = m->nextPtr )->prevPtr = m->prevPtr;
    else lPtr = cPtr = m->prevPtr;
    if ( m->prevPtr ) m->prevPtr->nextPtr = m->nextPtr;
    else fPtr = cPtr;
    --listSize;
    if ( list.cPtr )
    {
        m->nextPtr = list.cPtr;
        m->prevPtr = list.cPtr->prevPtr;
        list.cPtr->prevPtr = m;
        if ( m->prevPtr ) m->prevPtr->nextPtr = m;
        else list.fPtr = m;
    }
    else
    {
        m->prevPtr = m->nextPtr = 0;
        list.fPtr = list.lPtr = m;
    }
    list.cPtr = m;
    ++list.listSize;
}

void ShevList::movCurAftCur ( ShevList & list )
{
    if ( !cPtr ) return;
    ShevItem * m = cPtr;
    if ( m->nextPtr ) ( cPtr = m->nextPtr )->prevPtr = m->prevPtr;
    else lPtr = cPtr = m->prevPtr;
    if ( m->prevPtr ) m->prevPtr->nextPtr = m->nextPtr;
    else fPtr = cPtr;
    --listSize;
    if ( list.cPtr )
    {
        m->prevPtr = list.cPtr;
        m->nextPtr = list.cPtr->nextPtr;
        list.cPtr->nextPtr = m;
        if ( m->nextPtr ) m->nextPtr->prevPtr = m;
        else list.lPtr = m;
    }
    else
    {
        m->prevPtr = m->nextPtr = 0;
        list.fPtr = list.lPtr = m;
    }
    list.cPtr = m;
    ++list.listSize;
}

void ShevList::movCurAftLas ( ShevList & list )
{
    if ( !cPtr ) return;
    ShevItem * m = cPtr;
    if ( m->nextPtr ) ( cPtr = m->nextPtr )->prevPtr = m->prevPtr;
    else lPtr = cPtr = m->prevPtr;
    if ( m->prevPtr ) m->prevPtr->nextPtr = m->nextPtr;
    else fPtr = cPtr;
    --listSize;
    if ( list.lPtr )
    {
        m->prevPtr = list.lPtr;
        list.lPtr = list.lPtr->nextPtr = m;
    }
    else
    {
        m->prevPtr = 0;
        list.cPtr = list.fPtr = list.lPtr = m;
    }
    m->nextPtr = 0;
    ++list.listSize;
}

void ShevList::movAllBefFir ( ShevList & list )
{
    if ( ! cPtr || this == &list ) return;
    if ( list.cPtr )
    {
        lPtr->nextPtr = list.fPtr;
        list.fPtr->prevPtr = lPtr;
        list.fPtr = fPtr;
        list.listSize += listSize;
    }
    else
    {
        list.cPtr = cPtr;
        list.fPtr = fPtr;
        list.lPtr = lPtr;
        list.listSize = listSize;
    }
    fPtr = cPtr = lPtr = 0;
    listSize = 0;
}

void ShevList::movAllBefCur ( ShevList & list )
{
    if ( ! cPtr || this == &list ) return;
    if ( list.cPtr )
    {
        fPtr->prevPtr = list.cPtr->prevPtr;
        lPtr->nextPtr = list.cPtr;
        list.cPtr->prevPtr = lPtr;
        if ( fPtr->prevPtr )
             fPtr->prevPtr->nextPtr = fPtr;
        else list.fPtr = fPtr;
        list.listSize += listSize;
    }
    else
    {
        list.fPtr = fPtr;
        list.lPtr = lPtr;
        list.listSize = listSize;
    }
    list.cPtr = cPtr;
    fPtr = cPtr = lPtr = 0;
    listSize = 0;
}

void ShevList::movAllAftCur ( ShevList & list )
{
    if ( ! cPtr || this == &list ) return;
    if ( list.cPtr )
    {
        fPtr->prevPtr = list.cPtr;
        lPtr->nextPtr = list.cPtr->nextPtr;
        list.cPtr->nextPtr = fPtr;
        if ( lPtr->nextPtr )
             lPtr->nextPtr->prevPtr = lPtr;
        else list.lPtr = lPtr;
        list.listSize += listSize;
    }
    else
    {
        list.fPtr = fPtr;
        list.lPtr = lPtr;
        list.listSize = listSize;
    }
    list.cPtr = cPtr;
    fPtr = cPtr = lPtr = 0;
    listSize = 0;
}

void ShevList::movAllAftLas ( ShevList & list )
{
    if ( ! cPtr || this == &list ) return;
    if ( list.cPtr )
    {
        fPtr->prevPtr = list.lPtr;
        list.lPtr->nextPtr = fPtr;
        list.lPtr = lPtr;
        list.listSize += listSize;
    }
    else
    {
        list.cPtr = cPtr;
        list.fPtr = fPtr;
        list.lPtr = lPtr;
        list.listSize = listSize;
    }
    fPtr = cPtr = lPtr = 0;
    listSize = 0;
}

//**************************** 18.04.2006 **********************************
//
//               
//
//**************************** 16.01.2007 **********************************

void ShevList::mov123AftLas ( Predicate1 & pre, ShevList & list )
{
    if ( ! fPtr ) return;
    ShevItem * fi = fPtr;
    while ( ! pre ( fi ) )
    {
        fi = fi->nextPtr;
        if ( fi == 0 ) return;
    }

    ShevItem * li = fi;
    lPtr = fi->prevPtr;
    if ( lPtr == 0 ) fPtr = 0;
    ShevItem * p = fi->nextPtr;
    if ( this == &list )
    {
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->prevPtr != li )
                {
                    li->nextPtr = p;
                    p->prevPtr = li;
                }
                li = p;
            }
            else
            {
                if ( p->prevPtr != lPtr )
                {
                    if ( lPtr == 0 )
                    {
                        fPtr = p;
                        fPtr->prevPtr = 0;
                    }
                    else
                    {
                        lPtr->nextPtr = p;
                        p->prevPtr = lPtr;
                    }
                }
                lPtr = p;
            }
            p = p->nextPtr;
        }
        li->nextPtr = 0;
        if ( lPtr != 0 )
        {
            lPtr->nextPtr = fi;
            fi->prevPtr = lPtr;
        }
        else
        {
            fPtr = fi;
        }
        lPtr = li;
    }
    else
    {
        unsigned int count = 1;
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->prevPtr != li )
                {
                    li->nextPtr = p;
                    p->prevPtr = li;
                }
                li = p;
                ++count;
            }
            else
            {
                if ( p->prevPtr != lPtr )
                {
                    if ( lPtr == 0 )
                    {
                        fPtr = p;
                        fPtr->prevPtr = 0;
                    }
                    else
                    {
                        lPtr->nextPtr = p;
                        p->prevPtr = lPtr;
                    }
                }
                lPtr = p;
            }
            p = p->nextPtr;
        }
        li->nextPtr = 0;
        if ( ( cPtr = lPtr ) != 0 ) fPtr->prevPtr = lPtr->nextPtr = 0;
        listSize -= count;

        if ( list.lPtr != 0 )
        {
            list.lPtr->nextPtr = fi;
            fi->prevPtr = list.lPtr;
        }
        else
        {
            fi->prevPtr = 0;
            list.cPtr = list.fPtr = fi;
        }
        list.lPtr = li;
        list.listSize += count;
    }
}

void ShevList::mov321AftLas ( Predicate1 & pre, ShevList & list )
{
    if ( ! lPtr ) return;
    ShevItem * li = lPtr;
    while ( ! pre ( li ) )
    {
        li = li->prevPtr;
        if ( li == 0 ) return;
    }

    ShevItem * fi = li;
    fPtr = li->nextPtr;
    if ( fPtr == 0 ) lPtr = 0;
    ShevItem * p = li->prevPtr;
    if ( this == &list )
    {
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->nextPtr != fi )
                {
                    fi->prevPtr = p;
                    p->nextPtr = fi;
                }
                fi = p;
            }
            else
            {
                if ( p->nextPtr != fPtr )
                {
                    if ( fPtr == 0 )
                    {
                        lPtr = p;
                        lPtr->nextPtr = 0;
                    }
                    else
                    {
                        fPtr->prevPtr = p;
                        p->nextPtr = fPtr;
                    }
                }
                fPtr = p;
            }
            p = p->prevPtr;
        }
        li->nextPtr = 0;
        if ( lPtr != 0 )
        {
            lPtr->nextPtr = fi;
            fi->prevPtr = lPtr;
            fPtr->prevPtr = 0;
        }
        else
        {
            fPtr = fi;
        }
        lPtr = li;
    }
    else
    {
        unsigned int count = 1;
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->nextPtr != fi )
                {
                    fi->prevPtr = p;
                    p->nextPtr = fi;
                }
                fi = p;
                ++count;
            }
            else
            {
                if ( p->nextPtr != fPtr )
                {
                    if ( fPtr == 0 )
                    {
                        lPtr = p;
                        lPtr->nextPtr = 0;
                    }
                    else
                    {
                        fPtr->prevPtr = p;
                        p->nextPtr = fPtr;
                    }
                }
                fPtr = p;
            }
            p = p->prevPtr;
        }
        li->nextPtr = 0;
        if ( ( cPtr = lPtr ) != 0 ) fPtr->prevPtr = lPtr->nextPtr = 0;
        listSize -= count;

        if ( list.lPtr != 0 )
        {
            list.lPtr->nextPtr = fi;
            fi->prevPtr = list.lPtr;
        }
        else
        {
            fi->prevPtr = 0;
            list.cPtr = list.fPtr = fi;
        }
        list.lPtr = li;
        list.listSize += count;
    }
}

//**************************** 18.04.2006 **********************************
//
//               
//
//**************************** 16.01.2007 **********************************


void ShevList::mov123BefFir ( Predicate1 & pre, ShevList & list )
{
    if ( ! fPtr ) return;
    ShevItem * fi = fPtr;
    while ( ! pre ( fi ) )
    {
        fi = fi->nextPtr;
        if ( fi == 0 ) return;
    }

    ShevItem * li = fi;
    lPtr = fi->prevPtr;
    if ( lPtr == 0 ) fPtr = 0;
    ShevItem * p = fi->nextPtr;
    if ( this == &list )
    {
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->prevPtr != li )
                {
                    li->nextPtr = p;
                    p->prevPtr = li;
                }
                li = p;
            }
            else
            {
                if ( p->prevPtr != lPtr )
                {
                    if ( lPtr == 0 )
                    {
                        fPtr = p;
                        fPtr->prevPtr = 0;
                    }
                    else
                    {
                        lPtr->nextPtr = p;
                        p->prevPtr = lPtr;
                    }
                }
                lPtr = p;
            }
            p = p->nextPtr;
        }
        fi->prevPtr = 0;
        if ( fPtr != 0 )
        {
            fPtr->prevPtr = li;
            li->nextPtr = fPtr;
            lPtr->nextPtr = 0;
        }
        else
        {
            lPtr = li;
        }
        fPtr = fi;
    }
    else
    {
        unsigned int count = 1;
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->prevPtr != li )
                {
                    li->nextPtr = p;
                    p->prevPtr = li;
                }
                li = p;
                ++count;
            }
            else
            {
                if ( p->prevPtr != lPtr )
                {
                    if ( lPtr == 0 )
                    {
                        fPtr = p;
                        fPtr->prevPtr = 0;
                    }
                    else
                    {
                        lPtr->nextPtr = p;
                        p->prevPtr = lPtr;
                    }
                }
                lPtr = p;
            }
            p = p->nextPtr;
        }
        fi->prevPtr = 0;
        if ( ( cPtr = lPtr ) != 0 ) fPtr->prevPtr = lPtr->nextPtr = 0;
        listSize -= count;

        if ( list.fPtr != 0 )
        {
            list.fPtr->prevPtr = li;
            li->nextPtr = list.fPtr;
        }
        else
        {
            li->nextPtr = 0;
            list.cPtr = list.lPtr = li;
        }
        list.fPtr = fi;
        list.listSize += count;
    }
}

void ShevList::mov321BefFir ( Predicate1 & pre, ShevList & list )
{
    if ( ! lPtr ) return;
    ShevItem * li = lPtr;
    while ( ! pre ( li ) )
    {
        li = li->prevPtr;
        if ( li == 0 ) return;
    }

    ShevItem * fi = li;
    fPtr = li->nextPtr;
    if ( fPtr == 0 ) lPtr = 0;
    ShevItem * p = li->prevPtr;
    if ( this == &list )
    {
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->nextPtr != fi )
                {
                    fi->prevPtr = p;
                    p->nextPtr = fi;
                }
                fi = p;
            }
            else
            {
                if ( p->nextPtr != fPtr )
                {
                    if ( fPtr == 0 )
                    {
                        lPtr = p;
                        lPtr->nextPtr = 0;
                    }
                    else
                    {
                        fPtr->prevPtr = p;
                        p->nextPtr = fPtr;
                    }
                }
                fPtr = p;
            }
            p = p->prevPtr;
        }
        fi->prevPtr = 0;
        if ( fPtr != 0 )
        {
            fPtr->prevPtr = li;
            li->nextPtr = fPtr;
        }
        else
        {
            lPtr = li;
        }
        fPtr = fi;
    }
    else
    {
        unsigned int count = 1;
        while ( p != 0 )
        {
            if ( pre ( p ) )
            {
                if ( p->nextPtr != fi )
                {
                    fi->prevPtr = p;
                    p->nextPtr = fi;
                }
                fi = p;
                ++count;
            }
            else
            {
                if ( p->nextPtr != fPtr )
                {
                    if ( fPtr == 0 )
                    {
                        lPtr = p;
                        lPtr->nextPtr = 0;
                    }
                    else
                    {
                        fPtr->prevPtr = p;
                        p->nextPtr = fPtr;
                    }
                }
                fPtr = p;
            }
            p = p->prevPtr;
        }
        fi->prevPtr = 0;
        if ( ( cPtr = fPtr ) != 0 ) fPtr->prevPtr = lPtr->nextPtr = 0;
        listSize -= count;

        if ( list.fPtr != 0 )
        {
            list.fPtr->prevPtr = li;
            li->nextPtr = list.fPtr;
        }
        else
        {
            li->nextPtr = 0;
            list.cPtr = list.lPtr = li;
        }
        list.fPtr = fi;
        list.listSize += count;
    }
}

//    

ShevItem * ShevList::outFir()
{
    if ( ! cPtr ) return cPtr;
    ShevItem * v = fPtr;
    if ( fPtr == lPtr )
    {
        fPtr = cPtr = lPtr = 0;
        listSize = 0;
    }
    else
    {
        ( fPtr = v -> nextPtr ) -> prevPtr = 0;
        if ( cPtr == v ) cPtr = fPtr;
        --listSize;
    }
    return v;
}

ShevItem * ShevList::outCur()
{
    if ( ! cPtr ) return cPtr;
    ShevItem * v = cPtr;
    if ( v -> nextPtr ) ( cPtr = v -> nextPtr ) -> prevPtr = v -> prevPtr;
    else lPtr = cPtr = v -> prevPtr;
    if ( v -> prevPtr ) v -> prevPtr -> nextPtr = v -> nextPtr;
    else fPtr = cPtr;
    --listSize;
    return v;
}

ShevItem * ShevList::outLas()
{
    if ( ! cPtr ) return cPtr;
    ShevItem * v = lPtr;
    if ( fPtr == lPtr )
    {
        fPtr = cPtr = lPtr = 0;
        listSize = 0;
    }
    else
    {
        ( lPtr = v -> prevPtr ) -> nextPtr = 0;
        if ( cPtr == v ) cPtr = lPtr;
        --listSize;
    }
    return v;
}

void ShevList::invert ()
{
    ShevItem * temp, * item;

    for ( item = fPtr; item; item = temp )
    {
        temp = item->nextPtr;
        item->nextPtr = item->prevPtr;
        item->prevPtr = temp;
    }

    temp = lPtr;
    lPtr = fPtr;
    fPtr = temp;
}

void ShevList::sort123 ()
{
    if ( listSize == 2 )
    {
        if ( fPtr->info > lPtr->info )
        {
            cPtr = fPtr;
            fPtr = lPtr;
            lPtr = cPtr;
            fPtr->nextPtr = lPtr;
            lPtr->prevPtr = fPtr;
            fPtr->prevPtr = lPtr->nextPtr = 0;
        }
        return;
    }
    if ( listSize < 2 ) return;
    ShevList list;
    movLasAftLas ( listSize / 2, list );
    sort123 ();
    list.sort123 ();
    ShevItem * ptr;
    if ( fPtr->info <= list.fPtr->info )
    {
        ptr = fPtr;
        cPtr = ptr->nextPtr;
        list.cPtr = list.fPtr;
    }
    else
    {
        ptr = list.fPtr;
        list.cPtr = ptr->nextPtr;
        cPtr = fPtr;
        fPtr = ptr;
    }
    for (;;)
    {
        if ( ! cPtr )
        {
            list.cPtr->prevPtr = ptr;
            ptr->nextPtr = list.cPtr;
            cPtr = lPtr = list.lPtr;
            break;
        }
        if ( ! list.cPtr )
        {
            cPtr->prevPtr = ptr;
            ptr->nextPtr = cPtr;
            break;
        }
        if ( cPtr->info <= list.cPtr->info )
        {
            ptr->nextPtr = cPtr;
            cPtr->prevPtr = ptr;
            ptr = cPtr;
            cPtr = ptr->nextPtr;
        }
        else
        {
            ptr->nextPtr = list.cPtr;
            list.cPtr->prevPtr = ptr;
            ptr = list.cPtr;
            list.cPtr = ptr->nextPtr;
        }
    }
    list.fPtr = list.cPtr = list.lPtr = 0;
    listSize += list.listSize;
}

void ShevList::sort321 ()
{
    if ( listSize == 2 )
    {
        if ( fPtr->info < lPtr->info )
        {
            cPtr = fPtr;
            fPtr = lPtr;
            lPtr = cPtr;
            fPtr->nextPtr = lPtr;
            lPtr->prevPtr = fPtr;
            fPtr->prevPtr = lPtr->nextPtr = 0;
        }
        return;
    }
    if ( listSize < 2 ) return;
    ShevList list;
    movLasAftLas ( listSize / 2, list );
    sort321 ();
    list.sort321 ();
    ShevItem * ptr;
    if ( fPtr->info >= list.fPtr->info )
    {
        ptr = fPtr;
        cPtr = ptr->nextPtr;
        list.cPtr = list.fPtr;
    }
    else
    {
        ptr = list.fPtr;
        list.cPtr = ptr->nextPtr;
        cPtr = fPtr;
        fPtr = ptr;
    }
    for (;;)
    {
        if ( ! cPtr )
        {
            list.cPtr->prevPtr = ptr;
            ptr->nextPtr = list.cPtr;
            cPtr = lPtr = list.lPtr;
            break;
        }
        if ( ! list.cPtr )
        {
            cPtr->prevPtr = ptr;
            ptr->nextPtr = cPtr;
            break;
        }
        if ( cPtr->info >= list.cPtr->info )
        {
            ptr->nextPtr = cPtr;
            cPtr->prevPtr = ptr;
            ptr = cPtr;
            cPtr = ptr->nextPtr;
        }
        else
        {
            ptr->nextPtr = list.cPtr;
            list.cPtr->prevPtr = ptr;
            ptr = list.cPtr;
            list.cPtr = ptr->nextPtr;
        }
    }
    list.fPtr = list.cPtr = list.lPtr = 0;
    listSize += list.listSize;
}

void ShevList::sort ( bool (*f) ( const ShevItem *, const ShevItem * ) )
{
    if ( listSize == 2 )
    {
        if ( ! f ( fPtr, lPtr ) )
        {
            cPtr = fPtr;
            fPtr = lPtr;
            lPtr = cPtr;
            fPtr->nextPtr = lPtr;
            lPtr->prevPtr = fPtr;
            fPtr->prevPtr = lPtr->nextPtr = 0;
        }
        return;
    }
    if ( listSize < 2 ) return;
    ShevList list;
    movLasAftLas ( listSize / 2, list );
    sort ( f );
    list.sort ( f );
    ShevItem * ptr;
    if ( f ( fPtr, list.fPtr ) )
    {
        ptr = fPtr;
        cPtr = ptr->nextPtr;
        list.cPtr = list.fPtr;
    }
    else
    {
        ptr = list.fPtr;
        list.cPtr = ptr->nextPtr;
        cPtr = fPtr;
        fPtr = ptr;
    }
    for (;;)
    {
        if ( ! cPtr )
        {
            list.cPtr->prevPtr = ptr;
            ptr->nextPtr = list.cPtr;
            cPtr = lPtr = list.lPtr;
            break;
        }
        if ( ! list.cPtr )
        {
            cPtr->prevPtr = ptr;
            ptr->nextPtr = cPtr;
            break;
        }
        if ( f ( cPtr, list.cPtr ) )
        {
            ptr->nextPtr = cPtr;
            cPtr->prevPtr = ptr;
            ptr = cPtr;
            cPtr = ptr->nextPtr;
        }
        else
        {
            ptr->nextPtr = list.cPtr;
            list.cPtr->prevPtr = ptr;
            ptr = list.cPtr;
            list.cPtr = ptr->nextPtr;
        }
    }
    list.fPtr = list.cPtr = list.lPtr = 0;
    listSize += list.listSize;
}

void ShevList::sort ( Predicate2 & pre )
{
    if ( listSize == 2 )
    {
        if ( ! pre ( fPtr, lPtr ) )
        {
            cPtr = fPtr;
            fPtr = lPtr;
            lPtr = cPtr;
            fPtr->nextPtr = lPtr;
            lPtr->prevPtr = fPtr;
            fPtr->prevPtr = lPtr->nextPtr = 0;
        }
        return;
    }
    if ( listSize < 2 ) return;
    ShevList list;
    movLasAftLas ( listSize / 2, list );
    sort ( pre );
    list.sort ( pre );
    ShevItem * ptr;
    if ( pre ( fPtr, list.fPtr ) )
    {
        ptr = fPtr;
        cPtr = ptr->nextPtr;
        list.cPtr = list.fPtr;
    }
    else
    {
        ptr = list.fPtr;
        list.cPtr = ptr->nextPtr;
        cPtr = fPtr;
        fPtr = ptr;
    }
    for (;;)
    {
        if ( ! cPtr )
        {
            list.cPtr->prevPtr = ptr;
            ptr->nextPtr = list.cPtr;
            cPtr = lPtr = list.lPtr;
            break;
        }
        if ( ! list.cPtr )
        {
            cPtr->prevPtr = ptr;
            ptr->nextPtr = cPtr;
            break;
        }
        if ( pre ( cPtr, list.cPtr ) )
        {
            ptr->nextPtr = cPtr;
            cPtr->prevPtr = ptr;
            ptr = cPtr;
            cPtr = ptr->nextPtr;
        }
        else
        {
            ptr->nextPtr = list.cPtr;
            list.cPtr->prevPtr = ptr;
            ptr = list.cPtr;
            list.cPtr = ptr->nextPtr;
        }
    }
    list.fPtr = list.cPtr = list.lPtr = 0;
    listSize += list.listSize;
}

//      ?

bool ShevList::has ( const ShevItem * m ) const
{
    const ShevItem * t = fPtr;
    while ( t )
    {
        if ( t == m ) return true;
        t = t->nextPtr;
    }
    return false;
}

bool ShowList::has ( const ShevItem * m ) const
{
    const ShevItem * t = fPtr;
    while ( t )
    {
        if ( t == m ) return true;
        t = t->nextPtr;
    }
    return false;
}

//      info  i

void ShevList::setAllInfo ( int i )
{
    ShevItem * t = fPtr;
    while ( t )
    {
        t->info = i;
        t = t->nextPtr;
    }
}


//    :

void ShevList::for_each123 ( Alter & alt )
{
    ShevItem * t = fPtr;
    while ( t )
    {
        alt ( t );
        t = t->nextPtr;
    }
}

void ShevList::for_each321 ( Alter & alt )
{
    ShevItem * t = lPtr;
    while ( t )
    {
        alt ( t );
        t = t->prevPtr;
    }
}

//    :

void ShevList::for_some123 ( Predicate1 & pre, Alter & alt )
{
    ShevItem * t = fPtr;
    while ( t )
    {
        if ( pre ( t ) ) alt ( t );
        t = t->nextPtr;
    }
}

void ShevList::for_some321 ( Predicate1 & pre, Alter & alt )
{
    ShevItem * t = lPtr;
    while ( t )
    {
        if ( pre ( t ) ) alt ( t );
        t = t->prevPtr;
    }
}

void ShevList::makeCurFir()
{
    if ( fPtr == cPtr ) return;
    fPtr->prevPtr = lPtr;
    lPtr->nextPtr = fPtr;
    lPtr = cPtr->prevPtr;
    fPtr = cPtr;
    fPtr->prevPtr = lPtr->nextPtr = 0;
}

void ShevList::makeCurLas()
{
    if ( lPtr == cPtr ) return;
    fPtr->prevPtr = lPtr;
    lPtr->nextPtr = fPtr;
    fPtr = cPtr->nextPtr;
    lPtr = cPtr;
    fPtr->prevPtr = lPtr->nextPtr = 0;
}

void ShevList::swapCurAndLas()
{
    if ( cPtr == lPtr ) return;
    if ( cPtr == fPtr )
    {
        swapFirAndLas(); return;
    }
    ShevItem * t = cPtr;
    cPtr = lPtr;
    lPtr = t;
    if ( lPtr->nextPtr == cPtr )
    {
        lPtr->prevPtr->nextPtr = cPtr;
        cPtr->prevPtr = lPtr->prevPtr;
        cPtr->nextPtr = lPtr;
        lPtr->prevPtr = cPtr;
        lPtr->nextPtr = 0;
    }
    else
    {
        t = cPtr->prevPtr;
        cPtr->prevPtr = lPtr->prevPtr;
        lPtr->prevPtr = t;
        t = cPtr->nextPtr;
        cPtr->nextPtr = lPtr->nextPtr;
        lPtr->nextPtr = t;
        cPtr->prevPtr->nextPtr = cPtr;
        cPtr->nextPtr->prevPtr = cPtr;
        lPtr->prevPtr->nextPtr = lPtr;
    }
}

void ShevList::swapCurAndFir()
{
    if ( cPtr == fPtr ) return;
    if ( cPtr == lPtr )
    {
        swapFirAndLas(); return;
    }
    ShevItem * t = cPtr;
    cPtr = fPtr;
    fPtr = t;
    if ( fPtr->prevPtr == cPtr )
    {
        fPtr->nextPtr->prevPtr = cPtr;
        cPtr->nextPtr = fPtr->nextPtr;
        cPtr->prevPtr = fPtr;
        fPtr->nextPtr = cPtr;
        fPtr->prevPtr = 0;
    }
    else
    {
        t = cPtr->prevPtr;
        cPtr->prevPtr = fPtr->prevPtr;
        fPtr->prevPtr = t;
        t = cPtr->nextPtr;
        cPtr->nextPtr = fPtr->nextPtr;
        fPtr->nextPtr = t;
        cPtr->nextPtr->prevPtr = cPtr;
        cPtr->prevPtr->nextPtr = cPtr;
        fPtr->nextPtr->prevPtr = fPtr;
    }
}

void ShevList::swapFirAndLas()
{
    if ( lPtr == fPtr ) return;
    ShevItem * t = fPtr;
    fPtr = lPtr;
    lPtr = t;
    if ( listSize == 2 )
    {
        fPtr->nextPtr = lPtr;
        lPtr->prevPtr = fPtr;
        fPtr->prevPtr = lPtr->nextPtr = 0;
    }
    else
    {
        fPtr->nextPtr = lPtr->nextPtr;
        lPtr->prevPtr = fPtr->prevPtr;
        fPtr->prevPtr = lPtr->nextPtr = 0;
        fPtr->nextPtr->prevPtr = fPtr;
        lPtr->prevPtr->nextPtr = lPtr;
    }
    if ( cPtr == fPtr ) cPtr = lPtr; else
    if ( cPtr == lPtr ) cPtr = fPtr;
}

void swap ( ShevList & dl1, ShevList & dl2 )
{
    ShevItem * t = dl1.cPtr;
    dl1.cPtr = dl2.cPtr;
    dl2.cPtr = t;
    t = dl1.fPtr;
    dl1.fPtr = dl2.fPtr;
    dl2.fPtr = t;
    t = dl1.lPtr;
    dl1.lPtr = dl2.lPtr;
    dl2.lPtr = t;
    unsigned int k = dl1.listSize;
    dl1.listSize = dl2.listSize;
    dl2.listSize = k;
}

void swapCurItems ( ShevList & dl1, ShevList & dl2 )
{
    if ( &dl1 == &dl2 ) return;
    if ( !dl1.cPtr || !dl2.cPtr ) return;
    if ( dl1.cPtr->prevPtr ) dl1.cPtr->prevPtr->nextPtr = dl2.cPtr;
    if ( dl1.cPtr->nextPtr ) dl1.cPtr->nextPtr->prevPtr = dl2.cPtr;
    if ( dl2.cPtr->prevPtr ) dl2.cPtr->prevPtr->nextPtr = dl1.cPtr;
    if ( dl2.cPtr->nextPtr ) dl2.cPtr->nextPtr->prevPtr = dl1.cPtr;
    ShevItem * t = dl1.cPtr->prevPtr;
    dl1.cPtr->prevPtr = dl2.cPtr->prevPtr;
    dl2.cPtr->prevPtr = t;
    t = dl1.cPtr->nextPtr;
    dl1.cPtr->nextPtr = dl2.cPtr->nextPtr;
    dl2.cPtr->nextPtr = t;
    if ( dl1.fPtr == dl1.cPtr ) dl1.fPtr = dl2.cPtr;
    if ( dl1.lPtr == dl1.cPtr ) dl1.lPtr = dl2.cPtr;
    if ( dl2.fPtr == dl2.cPtr ) dl2.fPtr = dl1.cPtr;
    if ( dl2.lPtr == dl2.cPtr ) dl2.lPtr = dl1.cPtr;
    t = dl1.cPtr;
    dl1.cPtr = dl2.cPtr;
    dl2.cPtr = t;
}

void ShevList::movFirBefFir ( unsigned int n, ShevList & list )
{
    if ( n == 0 || listSize == 0 || this == &list ) return;
    if ( n >= listSize )
    {
        movAllBefFir ( list );
        return;
    }
    cPtr = fPtr;
    for ( unsigned int i = n; i-- > 0; ) cPtr = cPtr->nextPtr;
    if ( list.listSize > 0 )
    {
        cPtr->prevPtr->nextPtr = list.fPtr;
        list.fPtr->prevPtr = cPtr->prevPtr;
        list.fPtr = fPtr;
        list.listSize += n;
    }
    else
    {
        list.lPtr = cPtr->prevPtr;
        list.lPtr->nextPtr = 0;
        list.cPtr = list.fPtr = fPtr;
        list.listSize = n;
    }
    fPtr = cPtr;
    fPtr->prevPtr = 0;
    listSize -= n;
}

void ShevList::movLasAftLas ( unsigned int n, ShevList & list )
{
    if ( n == 0 || listSize == 0 || this == &list ) return;
    if ( n >= listSize )
    {
        movAllAftLas ( list );
        return;
    }
    cPtr = lPtr;
    for ( unsigned int i = n; i-- > 0; ) cPtr = cPtr->prevPtr;
    if ( list.listSize > 0 )
    {
        cPtr->nextPtr->prevPtr = list.lPtr;
        list.lPtr->nextPtr = cPtr->nextPtr;
        list.lPtr = lPtr;
        list.listSize += n;
    }
    else
    {
        list.fPtr = cPtr->nextPtr;
        list.fPtr->prevPtr = 0;
        list.cPtr = list.lPtr = lPtr;
        list.listSize = n;
    }
    lPtr = cPtr;
    lPtr->nextPtr = 0;
    listSize -= n;
}

const char * ShevList::test() const
{
    if ( listSize == 0 )
    {
        if ( fPtr ) return "listSize == 0 && fPtr != 0";
        if ( cPtr ) return "listSize == 0 && cPtr != 0";
        if ( lPtr ) return "listSize == 0 && lPtr != 0";
        return 0;
    }
    if ( ! fPtr ) return "listSize > 0 && fPtr == 0";
    if ( ! cPtr ) return "listSize > 0 && cPtr == 0";
    if ( ! lPtr ) return "listSize > 0 && lPtr == 0";
    if ( fPtr->prevPtr ) return "fPtr->prevPtr != 0";
    if ( lPtr->nextPtr ) return "lPtr->nextPtr != 0";
    if ( listSize == 1 )
    {
        if ( fPtr != cPtr ) return "listSize == 1 && fPtr != cPtr";
        if ( cPtr != lPtr ) return "listSize == 1 && cPtr != lPtr";
        if ( lPtr != fPtr ) return "listSize == 1 && lPtr != fPtr";
        return 0;
    }
    if ( ! fPtr->nextPtr ) return "listSize > 1 && fPtr->nextPtr == 0";
    if ( ! lPtr->prevPtr ) return "listSize > 1 && lPtr->prevPtr == 0";
    if ( fPtr == lPtr    ) return "listSize > 1 && fPtr == lPtr";
    if ( listSize == 2 )
    {
        if ( fPtr != cPtr && cPtr != lPtr )
            return "listSize == 2 && fPtr != cPtr && cPtr != lPtr";
        if ( fPtr->nextPtr != lPtr )
            return "listSize == 2 && fPtr->nextPtr != lPtr";
        if ( lPtr->prevPtr != fPtr )
            return "listSize == 2 && lPtr->prevPtr != fPtr";
        return 0;
    }
    ShevItem * p = fPtr;
    for ( unsigned int i = 1; i < listSize; ++i )
    {
        if ( p == lPtr ) return "p == lPtr && i < listSize";
        if ( p->nextPtr == 0 ) return "p->nextPtr == 0 && p != lPtr";
        if ( p->nextPtr == fPtr ) return "p->nextPtr == fPtr";
        if ( p->nextPtr->prevPtr != p ) return "p->nextPtr->prevPtr != p";
        p = p->nextPtr;
    }
    if ( p != lPtr ) return "p != lPtr && i == listSize";
    return 0;
}

} // namespace SL;
