
#ifndef SHEV_LIST_H
#define SHEV_LIST_H

namespace SL
{

class ShevItem
{
    ShevItem * prevPtr;
    ShevItem * nextPtr;
//      :
    ShevItem ( ShevItem & );
    void operator = ( ShevItem & );
protected:
    virtual ~ShevItem ();
public:
    int info; //  
//   
    ShevItem () : prevPtr(0), nextPtr(0), info(-123456789) {}

    const ShevItem * nextItem () const { return nextPtr; }
    const ShevItem * prevItem () const { return prevPtr; }

    ShevItem * nextItem () { return nextPtr; }
    ShevItem * prevItem () { return prevPtr; }

friend class ShevList;
friend class ShowList;
friend void swapCurItems ( ShevList &, ShevList & );
};

//***************************************************************************//

class Predicate1
{
public:
    virtual bool operator () ( const ShevItem * ) = 0;
};

class InfoEQ : public Predicate1
{
public:
    const int i;
    explicit InfoEQ ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class InfoNE : public Predicate1
{
public:
    const int i;
    explicit InfoNE ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class InfoLT : public Predicate1
{
public:
    const int i;
    explicit InfoLT ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class InfoGT : public Predicate1
{
public:
    const int i;
    explicit InfoGT ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class InfoLE : public Predicate1
{
public:
    const int i;
    explicit InfoLE ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class InfoGE : public Predicate1
{
public:
    const int i;
    explicit InfoGE ( int p ) : i(p) {}
    virtual bool operator () ( const ShevItem * p );
};

class NotPredicate1 : public Predicate1
{
public:
    Predicate1 & pre;
    explicit NotPredicate1 ( Predicate1 & p ) : pre(p) {}
    virtual bool operator () ( const ShevItem * p );
};

inline NotPredicate1 operator ! ( Predicate1 & p ) { return NotPredicate1(p); }

class Or_Predicate1 : public Predicate1
{
public:
    Predicate1 & pre1;
    Predicate1 & pre2;
    Or_Predicate1 ( Predicate1 & p1, Predicate1 & p2 ) : pre1(p1), pre2(p2) {}
    virtual bool operator () ( const ShevItem * p );
};

inline Or_Predicate1 operator || ( Predicate1 & p1, Predicate1 & p2 ) { return Or_Predicate1(p1,p2); }

class AndPredicate1 : public Predicate1
{
public:
    Predicate1 & pre1;
    Predicate1 & pre2;
    AndPredicate1 ( Predicate1 & p1, Predicate1 & p2 ) : pre1(p1), pre2(p2) {}
    virtual bool operator () ( const ShevItem * p );
};

inline AndPredicate1 operator && ( Predicate1 & p1, Predicate1 & p2 ) { return AndPredicate1(p1,p2); }

//***************************************************************************//

class Alter
{
public:
    virtual void operator () ( ShevItem * );
};

extern Alter none;

class InfoAsn : public Alter
{
public:
    const int i;
    explicit InfoAsn ( int p ) : i(p) {}
    virtual void operator () ( ShevItem * p );
};

class InfoAdd : public Alter
{
public:
    const int i;
    explicit InfoAdd ( int p ) : i(p) {}
    virtual void operator () ( ShevItem * p );
};

class InfoSub : public Alter
{
public:
    const int i;
    explicit InfoSub ( int p ) : i(p) {}
    virtual void operator () ( ShevItem * p );
};

class InfoMul : public Alter
{
public:
    const int i;
    explicit InfoMul ( int p ) : i(p) {}
    virtual void operator () ( ShevItem * p );
};

class InfoDiv : public Alter
{
public:
    const int i;
    explicit InfoDiv ( int p ) : i(p) {}
    virtual void operator () ( ShevItem * p );
};

class If : public Alter
{
public:
    Predicate1 & pre;
    Alter & alt1, & alt2;
    If ( Predicate1 & p, Alter & a1, Alter & a2 ) : pre(p), alt1(a1), alt2(a2) {}
    virtual void operator () ( ShevItem * p );
};

//***************************************************************************//

class InfoActGen
{
public:
    InfoEQ operator == ( int i ) { return InfoEQ ( i ); }
    InfoNE operator != ( int i ) { return InfoNE ( i ); }
    InfoLT operator <  ( int i ) { return InfoLT ( i ); }
    InfoGT operator >  ( int i ) { return InfoGT ( i ); }
    InfoLE operator <= ( int i ) { return InfoLE ( i ); }
    InfoGE operator >= ( int i ) { return InfoGE ( i ); }

    InfoAsn operator  = ( int i ) { return InfoAsn ( i ); }
    InfoAdd operator += ( int i ) { return InfoAdd ( i ); }
    InfoSub operator -= ( int i ) { return InfoSub ( i ); }
    InfoMul operator *= ( int i ) { return InfoMul ( i ); }
    InfoDiv operator /= ( int i ) { return InfoDiv ( i ); }
};

extern InfoActGen Info;

//***************************************************************************//

class Predicate2
{
public:
    virtual bool operator () ( const ShevItem *, const ShevItem * ) = 0;
};

class Double123Sorter : public Predicate2
{
    const double * array;
public:
    explicit Double123Sorter ( const double * p ) : array ( p ) {} 
    bool operator () ( const ShevItem *, const ShevItem * );
};

class Double321Sorter : public Predicate2
{
    const double * array;
public:
    explicit Double321Sorter ( const double * p ) : array ( p ) {} 
    bool operator () ( const ShevItem *, const ShevItem * );
};

//***************************************************************************//

class ShevList : public ShevItem
{
    ShevItem * cPtr; //     
    ShevItem * fPtr; //     
    ShevItem * lPtr; //     
    unsigned int listSize; //    
//      :
    ShevList ( ShevList & );
    void operator = ( ShevList & );
public:
//   :
    ShevList () { cPtr = fPtr = lPtr = 0; listSize = 0; }
    ~ShevList ();
//    :
    unsigned int size () const { return listSize; }
//     :
    unsigned int count ( Predicate1 & ) const;
//     :
    const ShevItem * fir ( Predicate1 & ) const;
//     :
    const ShevItem * las ( Predicate1 & ) const;
//   ?
    bool curIsFir () const { return cPtr == fPtr; }
//   ?
    bool curIsLas () const { return cPtr == lPtr; }
//      ?
    bool has ( const ShevItem * ) const;
//     :
    const ShevItem * fir () const { return fPtr; }
    const ShevItem * cur () const { return cPtr; }
    const ShevItem * las () const { return lPtr; }
    ShevItem * fir () { return fPtr; }
    ShevItem * cur () { return cPtr; }
    ShevItem * las () { return lPtr; }
//    :
    ShevItem * top () { return cPtr = fPtr; }
//    :
    ShevItem * end () { return cPtr = lPtr; }
//    :
    ShevItem * prev () { return cPtr != fPtr ? cPtr = cPtr->prevPtr : 0; }
//    :
    ShevItem * next () { return cPtr != lPtr ? cPtr = cPtr->nextPtr : 0; }
//      :
    ShevItem * cprev () { return cPtr = (cPtr==fPtr) ? lPtr : cPtr->prevPtr; }
//      :
    ShevItem * cnext () { return cPtr = (cPtr==lPtr) ? fPtr : cPtr->nextPtr; }
//    :
    void jump ( ShevItem * m ) { cPtr = m; }
//       :
    bool findFir ( Predicate1 & );
//       :
    bool findLas ( Predicate1 & );
//       :
    void addBefFir ( ShevItem * );
//           :
    void addBefCur ( ShevItem * );
//           :
    void addAftCur ( ShevItem * );
//       :
    void addAftLas ( ShevItem * );
//      :
    void invert ();
    void makeCurFir ();
    void makeCurLas ();
    void swapCurAndFir ();
    void swapCurAndLas ();
    void swapFirAndLas ();
    void sort123 ();
    void sort321 ();
    void sort ( Predicate2 & );
    void sort ( bool (*f) ( const ShevItem *, const ShevItem * ) );
//        (    -
//      ,    .
//     . ):
    void movFirBefFir ( ShevList & list ) { list.addBefFir ( outFir() ); }
    void movFirBefCur ( ShevList & list ) { list.addBefCur ( outFir() ); }
    void movFirAftCur ( ShevList & list ) { list.addAftCur ( outFir() ); }
    void movFirAftLas ( ShevList & list ) { list.addAftLas ( outFir() ); }
    void movCurBefFir ( ShevList & list );
    void movCurBefCur ( ShevList & list );
    void movCurAftCur ( ShevList & list );
    void movCurAftLas ( ShevList & list );
    void movLasBefFir ( ShevList & list ) { list.addBefFir ( outLas() ); }
    void movLasBefCur ( ShevList & list ) { list.addBefCur ( outLas() ); }
    void movLasAftCur ( ShevList & list ) { list.addAftCur ( outLas() ); }
    void movLasAftLas ( ShevList & list ) { list.addAftLas ( outLas() ); }
//       :
    void movFirBefFir ( unsigned int, ShevList & );
    void movLasAftLas ( unsigned int, ShevList & );
    void movAllBefFir ( ShevList & );
    void movAllBefCur ( ShevList & );
    void movAllAftCur ( ShevList & );
    void movAllAftLas ( ShevList & );
    void mov123BefFir ( Predicate1 &, ShevList & );
    void mov123AftLas ( Predicate1 &, ShevList & );
    void mov321BefFir ( Predicate1 &, ShevList & );
    void mov321AftLas ( Predicate1 &, ShevList & );
//      info  i:
    void setAllInfo ( int i );
//    :
    void for_each123 ( Alter & );
    void for_each321 ( Alter & );
//    :
    void for_some123 ( Predicate1 &, Alter & );
    void for_some321 ( Predicate1 &, Alter & );
protected:
//    :
    ShevItem * outFir ();
    ShevItem * outCur ();
    ShevItem * outLas ();
    void outAll () { cPtr = fPtr = lPtr = 0; listSize = 0; }
public:
//    :
    void delFir () { delete outFir (); }
    void delCur () { delete outCur (); }
    void delLas () { delete outLas (); }
    void delAll ();
//  :
    ShevItem * delCurOrGoToNext ( bool del )
    {
        if ( ! del )
        {
            return next();
        }
        if ( curIsLas() )
        {
            delCur();
            return 0;
        }
        else
        {
            delCur();
            return cur();
        }
    }
//     (         0,     )
    const char * test () const;
//   :
    friend void swap ( ShevList &, ShevList & );
//   :
    friend void swapCurItems ( ShevList &, ShevList & );
//    
    friend class ShowList;
};

//***************************************************************************//

class ShowList
{
    const ShevItem *       cPtr; //     
    const ShevItem * const fPtr; //     
    const ShevItem * const lPtr; //     
    const unsigned int listSize; //    
public:
    const int              info; //  

    ShowList ( const ShevList & p ) : cPtr(p.cPtr), fPtr(p.fPtr), lPtr(p.lPtr), listSize(p.listSize), info(p.info) {}
//    :
    unsigned int size () const { return listSize; }
//     
    unsigned int count ( Predicate1 & ) const;
//     :
    const ShevItem * fir ( Predicate1 & ) const;
//     :
    const ShevItem * las ( Predicate1 & ) const;
//   ?
    bool curIsFir () const { return cPtr == fPtr; }
//   ?
    bool curIsLas () const { return cPtr == lPtr; }
//      ?
    bool has ( const ShevItem * ) const;
//     
    const ShevItem * fir () const { return fPtr; }
    const ShevItem * cur () const { return cPtr; }
    const ShevItem * las () const { return lPtr; }
//    
    const ShevItem * top () { return cPtr = fPtr; }
//    
    const ShevItem * end () { return cPtr = lPtr; }
//    
    const ShevItem * prev () { return cPtr != fPtr ? cPtr = cPtr->prevPtr : 0; }
//    
    const ShevItem * next () { return cPtr != lPtr ? cPtr = cPtr->nextPtr : 0; }
//      
    const ShevItem * cprev () { return cPtr = (cPtr==fPtr) ? lPtr : cPtr->prevPtr; }
//      
    const ShevItem * cnext () { return cPtr = (cPtr==lPtr) ? fPtr : cPtr->nextPtr; }
//    
    void jump ( const ShevItem * m ) { cPtr = m; }
};

//***************************************************************************//

template <class T> class Item : public ShevItem
{
public:
    T d;
    Item () {}
    Item ( const T & t ) : d(t) {}

    Item * nextItem () { return (Item*) ShevItem::nextItem (); }
    Item * prevItem () { return (Item*) ShevItem::prevItem (); }

    const Item * nextItem () const { return (Item*) ShevItem::nextItem (); }
    const Item * prevItem () const { return (Item*) ShevItem::prevItem (); }
};

template <class T> class List : public ShevList
{
//      :
    List<T> ( List<T> & );
    void operator = ( List<T> & );
public:
    List<T>() {}

    const T * fir() const { return (const T *) ShevList::fir(); }
    const T * cur() const { return (const T *) ShevList::cur(); }
    const T * las() const { return (const T *) ShevList::las(); }

    const T * fir ( Predicate1 & pre ) const { return (const T *) ShevList::fir(pre); }
    const T * las ( Predicate1 & pre ) const { return (const T *) ShevList::las(pre); }

    T * fir ()  { return (T *) ShevList::fir (); }
    T * cur ()  { return (T *) ShevList::cur (); }
    T * las ()  { return (T *) ShevList::las (); }
    T * top ()  { return (T *) ShevList::top (); }
    T * end ()  { return (T *) ShevList::end (); }
    T * prev()  { return (T *) ShevList::prev(); }
    T * next()  { return (T *) ShevList::next(); }
    T * cprev() { return (T *) ShevList::cprev(); }
    T * cnext() { return (T *) ShevList::cnext(); }

    void jump      ( T * i ) { ShevList::jump     (i); }
    void addBefFir ( T * i ) { ShevList::addBefFir(i); }
    void addBefCur ( T * i ) { ShevList::addBefCur(i); }
    void addAftCur ( T * i ) { ShevList::addAftCur(i); }
    void addAftLas ( T * i ) { ShevList::addAftLas(i); }

    void movFirBefFir ( List<T> & list ) { ShevList::movFirBefFir ( list ); }
    void movFirBefCur ( List<T> & list ) { ShevList::movFirBefCur ( list ); }
    void movFirAftCur ( List<T> & list ) { ShevList::movFirAftCur ( list ); }
    void movFirAftLas ( List<T> & list ) { ShevList::movFirAftLas ( list ); }
    void movCurBefFir ( List<T> & list ) { ShevList::movCurBefFir ( list ); }
    void movCurBefCur ( List<T> & list ) { ShevList::movCurBefCur ( list ); }
    void movCurAftCur ( List<T> & list ) { ShevList::movCurAftCur ( list ); }
    void movCurAftLas ( List<T> & list ) { ShevList::movCurAftLas ( list ); }
    void movLasBefFir ( List<T> & list ) { ShevList::movLasBefFir ( list ); }
    void movLasBefCur ( List<T> & list ) { ShevList::movLasBefCur ( list ); }
    void movLasAftCur ( List<T> & list ) { ShevList::movLasAftCur ( list ); }
    void movLasAftLas ( List<T> & list ) { ShevList::movLasAftLas ( list ); }

    void movFirBefFir ( unsigned int n, List<T> & list ) { ShevList::movFirBefFir ( n, list ); }
    void movLasAftLas ( unsigned int n, List<T> & list ) { ShevList::movLasAftLas ( n, list ); }

    void movAllBefFir ( List<T> & list ) { ShevList::movAllBefFir ( list ); }
    void movAllBefCur ( List<T> & list ) { ShevList::movAllBefCur ( list ); }
    void movAllAftCur ( List<T> & list ) { ShevList::movAllAftCur ( list ); }
    void movAllAftLas ( List<T> & list ) { ShevList::movAllAftLas ( list ); }

    T * delCurOrGoToNext ( bool del )
    {
        if ( ! del )
        {
            return next();
        }
        if ( curIsLas() )
        {
            delCur();
            return 0;
        }
        else
        {
            delCur();
            return cur();
        }
    }
};

template <class T> class Show : public ShowList
{
public:
    Show ( const List<T> & p ) : ShowList ( p ) {}
//     :
    const T * fir () const { return (const T *) ShowList::fir(); }
    const T * cur () const { return (const T *) ShowList::cur(); }
    const T * las () const { return (const T *) ShowList::las(); }
//     :
    const T * fir ( Predicate1 & p ) const { return (const T *) ShowList::fir(); }
//     :
    const T * las ( Predicate1 & p ) const { return (const T *) ShowList::las(); }
//    
    const T * top () { return (const T *) ShowList::top(); }
//    
    const T * end () { return (const T *) ShowList::end(); }
//    
    const T * prev () { return (const T *) ShowList::prev(); }
//    
    const T * next () { return (const T *) ShowList::next(); }
//      
    const T * cprev () { return (const T *) ShowList::cprev(); }
//      
    const T * cnext () { return (const T *) ShowList::cnext(); }
//    
    void jump ( const T * m ) { ShowList::jump ( m ); }
};

} // namespace SL

using namespace SL;

#endif
