
#include "ShevArray.h"
#include "lex_analyse.h"

//  
LexReturn lex_analyse ( nat32 size, const char8 * source, Suite<Token> & result )
{
    nat32 nlex = 0;         //  
    for ( nat32 i = 0; i < size; ++i )
    {
        Token token;
        const char8 c = source[i];
        if ( ( c >= 'a' && c <= 'z' ) || ( c >= 'A' && c <= 'Z' ) || c == '_' ) // 
        {
            if ( nlex > 0 )
            {
                Token & last = result[nlex-1];
                if ( last.type == tt_name )
                {
                    last.size += 1;
                    continue;
                }
//    
                if ( *last.text == '0' && last.size == 1 && c == 'b' )
                {
                    last.type = tt_bin;
                    if ( i < size-1 )
                    {
                        const char8 t = source[i+1];
                        if ( t == '0' || t == '1' )
                        {
                            last.size += 2;
                            ++i;
                            continue;
                        }
                        last.size += 1;
                    }
                    last.size += 1;
                    return lr_bad_bin;
                }
                if ( last.type == tt_bin )
                {
                    last.size += 1;
                    return lr_bad_bin;
                }
//    
                if ( *last.text == '0' && last.size == 1 && c == 'x' )
                {
                    last.type = tt_hex;
                    if ( i < size-1 )
                    {
                        const char8 t = source[i+1];
                        if ( ( t >= '0' && t <= '9' ) || ( t >= 'A' && t <= 'F' ) )
                        {
                            last.size += 2;
                            ++i;
                            continue;
                        }
                        last.size += 1;
                    }
                    last.size += 1;
                    return lr_bad_hex;
                }
                if ( last.type == tt_hex )
                {
                    if ( c >= 'A' && c <= 'F' )
                    {
                        last.size += 1;
                        continue;
                    }
                    last.size += 1;
                    return lr_bad_hex;
                }
//      
                if ( last.type == tt_int || last.type == tt_real )
                {
                    if ( c == 'e' )
                    {
                        last.type = tt_exp;
                        if ( i < size-1 )
                        {
                            const char8 t = source[i+1];
                            if ( t >= '0' && t <= '9' )
                            {
                                last.size += 2;
                                ++i;
                                continue;
                            }
                            if ( t == '+' || t == '-' )
                            {
                                if ( i < size-2 )
                                {
                                    const char8 t = source[i+2];
                                    if ( t >= '0' && t <= '9' )
                                    {
                                        last.size += 3;
                                        i += 2;
                                        continue;
                                    }
                                }
                            }
                            last.size += 1;
                        }
                    }
                    last.size += 1;
                    return lr_bad_num;
                }
            }
            token.type = tt_name;
        }
        else
        if ( c >= '0' && c <= '9' ) // 
        {
            if ( nlex > 0 )
            {
                Token & last = result[nlex-1];
                switch ( last.type )
                {
                case tt_name:
                case tt_int:
                case tt_bin:
                case tt_hex:
                case tt_real:
                    last.size += 1;
                    continue;
                }
            }
            token.type = tt_int;
        }
        else
        {
            switch ( c )
            {
            case '\t':
            case '\n':
            case '\v':
            case '\r':
            case ' ': 
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_comment )
                    {
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_comment; break;
            case '!': token.type = tt_exclamation; break;
            case '#': token.type = tt_sharp; break;
            case '"':  //    
                    {
                        nat32 j = i + 1;
                        for (;;)
                        {
                            if ( j >= size )
                            {
                                return lr_no_quote2;
                            }
                            if ( source[j] == '"' && source[j-1] != '\\') break;
                            ++j;
                        }
                        token.type = tt_quote2;
                        token.text = & source[i];
                        token.size = j - i + 1;
                        result.addAftLas ( token );
                        ++nlex;
                        i = j;
                        continue;
                    }
            case '%': token.type = tt_percent; break;
            case '&': token.type = tt_amper; break;
            case '\'':  //    
                    {
                        nat32 j = i + 1;
                        for (;;)
                        {
                            if ( j >= size )
                            {
                                return lr_no_quote1;
                            }
                            if ( source[j] == '\'' && source[j-1] != '\\') break;
                            ++j;
                        }
                        token.type = tt_quote1;
                        token.text = & source[i];
                        token.size = j - i + 1;
                        result.addAftLas ( token );
                        ++nlex;
                        i = j;
                        continue;
                    }
            case '(': token.type = tt_parenthesis_open; break;
            case ')': token.type = tt_parenthesis_close; break;
            case '*': token.type = tt_asterisk; break;
            case '+':
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_plus )
                    {
                        last.type = tt_plus_plus;
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_plus; break;
            case ',': token.type = tt_comma; break;
            case '-': 
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_minus )
                    {
                        last.type = tt_minus_minus;
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_minus; break;
            case '.': 
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_int )
                    {
                        last.type = tt_real;
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_dot; break;
            case '/': 
                if ( i < size-1 )
                {
                    const char8 t = source[i+1];
                    if ( t == '/' ) //  
                    {
                        nat32 j = i + 2;
                        for (;;)
                        {
                            if ( j >= size )
                            {
                                --j;
                                break;
                            }
                            if ( source[j] == '\n' ) break;
                            ++j;
                        }
                        if ( nlex > 0 )
                        {
                            Token & last = result[nlex-1];
                            if ( last.type == tt_comment )
                            {
                                last.size += j - i + 1;
                                i = j;
                                continue;
                            }
                        }
                        token.type = tt_comment;
                        token.text = & source[i];
                        token.size = j - i + 1;
                        result.addAftLas ( token );
                        ++nlex;
                        i = j;
                        continue;
                    }
                    else
                    if ( t == '*' ) //  
                    {
                        nat32 j = i + 2;
                        for (;;)
                        {
                            if ( j >= size )
                            {
                                return lr_open_comment;
                            }
                            if ( source[j] != '*' )
                            {
                                ++j;
                                continue;
                            }
                            if ( ++j == size )
                            {
                                return lr_open_comment;
                            }
                            if ( source[j] == '/' )
                            {
                                break;
                            }
                        }
                        if ( nlex > 0 )
                        {
                            Token & last = result[nlex-1];
                            if ( last.type == tt_comment )
                            {
                                last.size += j - i + 1;
                                i = j;
                                continue;
                            }
                        }
                        token.type = tt_comment;
                        token.text = & source[i];
                        token.size = j - i + 1;
                        result.addAftLas ( token );
                        ++nlex;
                        i = j;
                        continue;
                    }
                }
                token.type = tt_slash; break;
            case ':': token.type = tt_colon; break;
            case ';': token.type = tt_semicolon; break;
            case '<':
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_less )
                    {
                        last.type = tt_less_less;
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_less; break;
            case '=':  
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    switch ( last.type )
                    {
                    case tt_exclamation:
                        last.type = tt_exclamation_equal;   //  !=
                        last.size += 1;
                        continue;
                    case tt_percent:
                        last.type = tt_percent_equal;   //  %=
                        last.size += 1;
                        continue;
                    case tt_amper:
                        last.type = tt_amper_equal;   //  &=
                        last.size += 1;
                        continue;
                    case tt_asterisk:
                        last.type = tt_asterisk_equal;   //  *=
                        last.size += 1;
                        continue;
                    case tt_plus:
                        last.type = tt_plus_equal;   //  +=
                        last.size += 1;
                        continue;
                    case tt_minus:
                        last.type = tt_minus_equal;   //  -=
                        last.size += 1;
                        continue;
                    case tt_slash:
                        last.type = tt_slash_equal;   //  /=
                        last.size += 1;
                        continue;
                    case tt_less_less:
                        last.type = tt_less_less_equal;   //  <<=
                        last.size += 1;
                        continue;
                    case tt_less:
                        last.type = tt_less_equal;   //  <=
                        last.size += 1;
                        continue;
                    case tt_equal:
                        last.type = tt_equal_equal;   //  ==
                        last.size += 1;
                        continue;
                    case tt_greater_greater:
                        last.type = tt_greater_greater_equal;   //  >>=
                        last.size += 1;
                        continue;
                    case tt_greater:
                        last.type = tt_greater_equal;   //  >=
                        last.size += 1;
                        continue;
                    case tt_lid:
                        last.type = tt_lid_equal;   //  ^=
                        last.size += 1;
                        continue;
                    case tt_vertical:
                        last.type = tt_vertical_equal;   //  |=
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_equal; break;
            case '>': 
                if ( nlex > 0 )
                {
                    Token & last = result[nlex-1];
                    if ( last.type == tt_greater )
                    {
                        last.type = tt_greater_greater; // >>
                        last.size += 1;
                        continue;
                    }
                }
                token.type = tt_greater; break;
            case '?': token.type = tt_question; break;
            case '@': token.type = tt_at; break;
            case '[': token.type = tt_bracket_open; break;
            case ']': token.type = tt_bracket_close; break;
            case '^': token.type = tt_lid; break;
            case '{': token.type = tt_braces_open; break;
            case '|': token.type = tt_vertical; break;
            case '}': token.type = tt_braces_close; break;
            case '~': token.type = tt_tilde; break;
            default:
                {
                    token.type = tt_unknown;
                    token.text = & source[i];
                    token.size = 1;
                    result.addAftLas ( token );
                    ++nlex;
                    return lr_unknown_character;
                }
            }
        }
        token.text = & source[i];
        token.size = 1;
        result.addAftLas ( token );
        ++nlex;
    }
    return lr_ok;
}

//     
void keywords ( Suite<Token> & result )
{
    const nat32 nlex = result.size();
    for ( nat32 j = 0; j < nlex; ++j )
    {
        Token & token = result[j];
        if ( token.type != tt_name ) continue;
        switch ( token.size )
        {
        case 2:
            if ( token.text[0] == 'i' && token.text[1] == 'f' )
            {
                token.type = tt_if;
            }
            continue;
        case 3:
            if ( token.text[0] == 'f' && token.text[1] == 'o' && token.text[2] == 'r' )
            {
                token.type = tt_for;
            }
            continue;
        case 4:
            switch ( token.text[0] )
            {
            case 'a':
                if ( token.text[1] == 'u' && token.text[2] == 't' && token.text[3] == 'o' )
                {
                    token.type = tt_auto;
                }
                continue;
            case 'e':
                if ( token.text[1] == 'l' && token.text[2] == 's' && token.text[3] == 'e' )
                {
                    token.type = tt_else;
                }
                continue;
            case 'g':
                if ( token.text[1] == 'o' && token.text[2] == 't' && token.text[3] == 'o' )
                {
                    token.type = tt_goto;
                }
                continue;
            case 'i':
                if ( token.text[1] == 'n' && token.text[2] == 't' && token.text[3] == '8' )
                {
                    token.type = tt_int8;
                }
                continue;
            case 'l':
                if ( token.text[1] == 'o' && token.text[2] == 'g' && token.text[3] == '8' )
                {
                    token.type = tt_log8;
                }
                continue;
            case 'n':
                if ( token.text[1] == 'a' && token.text[2] == 't' && token.text[3] == '8' )
                {
                    token.type = tt_nat8;
                }
                continue;
            case 't':
                if ( token.text[1] == 'r' && token.text[2] == 'u' && token.text[3] == 'e' )
                {
                    token.type = tt_true;
                }
                continue;
            case 'v':
                if ( token.text[1] == 'o' && token.text[2] == 'i' && token.text[3] == 'd' )
                {
                    token.type = tt_void;
                }
                continue;
            }
        case 5:
            switch ( token.text[0] )
            {
            case 'b':
                if ( token.text[1] == 'r' && token.text[2] == 'e' && 
                     token.text[3] == 'a' && token.text[4] == 'k' )
                {
                    token.type = tt_break;
                }
                continue;
            case 'c':
                if ( token.text[1] == 'h' && token.text[2] == 'a' && 
                     token.text[3] == 'r' && token.text[4] == '8' )
                {
                    token.type = tt_char8;
                }
                else
                if ( token.text[1] == 'o' && token.text[2] == 'n' && 
                     token.text[3] == 's' && token.text[4] == 't' )
                {
                    token.type = tt_const;
                }
                continue;
            case 'f':
                if ( token.text[1] == 'a' && token.text[2] == 'l' && 
                     token.text[3] == 's' && token.text[4] == 'e' )
                {
                    token.type = tt_false;
                }
                continue;
            case 'i':
                if ( token.text[1] == 'n' && token.text[2] == 't' )
                {
                    if ( token.text[3] == '1' && token.text[4] == '6' )
                    {
                        token.type = tt_int16;
                    }
                    else
                    if ( token.text[3] == '3' && token.text[4] == '2' )
                    {
                        token.type = tt_int32;
                    }
                    else
                    if ( token.text[3] == '6' && token.text[4] == '4' )
                    {
                        token.type = tt_int64;
                    }
                }
                continue;
            case 'l':
                if ( token.text[4] == 'l' )
                {
                    if ( token.text[1] == 'a' && token.text[2] == 'b' && token.text[3] == 'e' )
                    {
                        token.type = tt_label;
                    }
                    if ( token.text[1] == 'o' && token.text[2] == 'c' && token.text[3] == 'a' )
                    {
                        token.type = tt_local;
                    }
                }
                continue;
            case 'n':
                if ( token.text[1] == 'a' && token.text[2] == 't' )
                {
                    if ( token.text[3] == '1' && token.text[4] == '6' )
                    {
                        token.type = tt_int16;
                    }
                    else
                    if ( token.text[3] == '3' && token.text[4] == '2' )
                    {
                        token.type = tt_int32;
                    }
                    else
                    if ( token.text[3] == '6' && token.text[4] == '4' )
                    {
                        token.type = tt_int64;
                    }
                }
                continue;
            }
            continue;
        case 6:
            switch ( token.text[0] )
            {
            case 'e':
                if ( token.text[1] == 'x' && token.text[2] == 't' && token.text[3] == 'e' &&
                     token.text[4] == 'r' && token.text[5] == 'n' )
                {
                    token.type = tt_extern;
                }
                continue;
            case 'g':
                if ( token.text[1] == 'l' && token.text[2] == 'o' && token.text[3] == 'b' &&
                     token.text[4] == 'a' && token.text[5] == 'l' )
                {
                    token.type = tt_global;
                }
                continue;
            case 'r':
                if ( token.text[1] == 'e' )
                {
                    if ( token.text[2] == 'a' && token.text[3] == 'l' )
                    {
                        if ( token.text[4] == '3' && token.text[5] == '2' )
                        {
                            token.type = tt_real32;
                        }
                        else
                        if ( token.text[4] == '6' && token.text[5] == '4' )
                        {
                            token.type = tt_real64;
                        }
                    }
                    else
                    if ( token.text[2] == 't' && token.text[3] == 'u' && 
                         token.text[4] == 'r' && token.text[5] == 'n' )
                    {
                        token.type = tt_return;
                    }
                }
                continue;
            case 's':
                if ( token.text[1] == 't' && token.text[2] == 'a' && token.text[3] == 't' &&
                     token.text[4] == 'i' && token.text[5] == 'c' )
                {
                    token.type = tt_static;
                }
                continue;
            }
        case 8:
            switch ( token.text[0] )
            {
            case 'c':
                if ( token.text[1] == 'o' && token.text[2] == 'n' && token.text[3] == 't' &&
                     token.text[4] == 'i' && token.text[5] == 'n' && token.text[6] == 'u' && token.text[7] == 'e' )
                {
                    token.type = tt_continue;
                }
                continue;
            }
            continue;
        }
    }
}