
#include "syntax.h"

#define SYNTAX_TEST

GSymbol::GSymbol ( const Rule * r, TokenType t ) : rule(r), type(t)
{
    while ( r != 0 )
    {
        const RuleAtom * atom = r->atom;
        if ( atom->symbol->rule == 0 )
        {
            list.addAftCur ( new ItemRAP ( atom ) );
            list.cur()->info = atom->symbol->type;
        }
        else
        {
            Show<ItemRAP> show ( atom->symbol->list );
            if ( show.top() )
            do
            {
                const ItemRAP * item = show.cur();
                list.addAftCur ( new ItemRAP ( atom ) );
                list.cur()->info = item->info;
            }
            while ( show.next() );
        }
        r = r->next;
    }
#ifdef SYNTAX_TEST
//   ,     
    if ( list.size() > 1 )
    {
        list.sort123();
        list.top();
        while ( list.next() )
        {
            const ShevItem * item = list.cur();
            if ( item->info == item->prevItem()->info )
            {
                bool ok = false;
            }
        }
    }
#endif
}

/////////////////////////////////////////////////////////////////////////////////////////

// 

const GSymbol s_name ( 0, tt_name );
const GSymbol s_int  ( 0, tt_int  ); //  
const GSymbol s_bin  ( 0, tt_bin  ); //  
const GSymbol s_hex  ( 0, tt_hex  ); //  
const GSymbol s_real ( 0, tt_real ); //    
const GSymbol s_exp  ( 0, tt_exp  ); //       

// 

const GSymbol s_exclamation       ( 0, tt_exclamation       ); // !   33
//    tt_sharp,               // #  35
const GSymbol s_percent           ( 0, tt_percent           ); // %  37
//    tt_amper,               // & 
const GSymbol s_parenthesis_open  ( 0, tt_parenthesis_open  ); // (   
const GSymbol s_parenthesis_close ( 0, tt_parenthesis_close ); // )   
const GSymbol s_asterisk          ( 0, tt_asterisk          ); // *  42
const GSymbol s_plus              ( 0, tt_plus              ); // + 
const GSymbol s_comma             ( 0, tt_comma             ); // , 
const GSymbol s_minus             ( 0, tt_minus             ); // - 
const GSymbol s_slash             ( 0, tt_slash             ); // /  
//    tt_colon,               // : 
const GSymbol s_semicolon         ( 0, tt_semicolon         ); // ;   
const GSymbol s_less              ( 0, tt_less              ); // < 
const GSymbol s_equal             ( 0, tt_equal             ); // = 
const GSymbol s_greater           ( 0, tt_greater           ); // >  62
//    tt_question,            // ?  63
//    tt_at,                  // @   64
const GSymbol s_bracket_open      ( 0, tt_bracket_open      ); // [    91
const GSymbol s_bracket_close     ( 0, tt_bracket_close     ); // ]    93
const GSymbol s_braces_open       ( 0, tt_braces_open       ); // {    123
const GSymbol s_vertical          ( 0, tt_vertical          ); // |   124
const GSymbol s_braces_close      ( 0, tt_braces_close      ); // }    125
const GSymbol s_tilde             ( 0, tt_tilde             ); // ~  126

//  

const GSymbol s_exclamation_equal ( 0, tt_exclamation_equal ); //  !=
const GSymbol s_percent_equal     ( 0, tt_percent_equal     ); //  %=
const GSymbol s_amper_equal       ( 0, tt_amper_equal       ); //  &=
const GSymbol s_asterisk_equal    ( 0, tt_asterisk_equal    ); //  *=
const GSymbol s_plus_plus         ( 0, tt_plus_plus         ); //  ++
const GSymbol s_plus_equal        ( 0, tt_plus_equal        ); //  +=
const GSymbol s_minus_minus       ( 0, tt_minus_minus       ); //  --
const GSymbol s_minus_equal       ( 0, tt_minus_equal       ); //  -=
const GSymbol s_slash_equal       ( 0, tt_slash_equal       ); //  /=
//    tt_less_less,           //  <<
//    tt_less_less_equal,     // <<=
const GSymbol s_less_equal        ( 0, tt_less_equal        ); //  <=
const GSymbol s_equal_equal       ( 0, tt_equal_equal       ); //  ==
//    tt_greater_greater,     //  >>
const GSymbol s_greater_equal     ( 0, tt_greater_equal     ); //  >=
//    tt_greater_greater_equal,// >>=
const GSymbol s_lid_equal         ( 0, tt_lid_equal         ); //  ^=
const GSymbol s_vertical_equal    ( 0, tt_vertical_equal    ); //  |=

//  

const GSymbol s_if ( 0, tt_if );

const GSymbol s_for ( 0, tt_for );

const GSymbol s_auto ( 0, tt_auto );
const GSymbol s_else ( 0, tt_else );
const GSymbol s_goto ( 0, tt_goto );
const GSymbol s_log8 ( 0, tt_log8 );
const GSymbol s_int8 ( 0, tt_int8 );
const GSymbol s_nat8 ( 0, tt_nat8 );
const GSymbol s_true ( 0, tt_true );
const GSymbol s_void ( 0, tt_void );

const GSymbol s_break ( 0, tt_break );
const GSymbol s_char8 ( 0, tt_char8 );
const GSymbol s_const ( 0, tt_const );
const GSymbol s_false ( 0, tt_false );
const GSymbol s_int16 ( 0, tt_int16 );
const GSymbol s_int32 ( 0, tt_int32 );
const GSymbol s_int64 ( 0, tt_int64 );
const GSymbol s_label ( 0, tt_label );
const GSymbol s_local ( 0, tt_local );
const GSymbol s_nat16 ( 0, tt_nat16 );
const GSymbol s_nat32 ( 0, tt_nat32 );
const GSymbol s_nat64 ( 0, tt_nat64 );

const GSymbol s_extern ( 0, tt_extern );
const GSymbol s_global ( 0, tt_global );
const GSymbol s_real32 ( 0, tt_real32 );
const GSymbol s_real64 ( 0, tt_real64 );
const GSymbol s_return ( 0, tt_return );
const GSymbol s_static ( 0, tt_static );

const GSymbol s_continue ( 0, tt_continue );

//  

const GSymbol s_eof ( 0, tt_eof );


/***************************************************************/
//
//                       
//
/***************************************************************/

namespace DataType
{
    const RuleAtom ra12 ( &s_char8, 0 );

    const Rule r12 ( &ra12, 0 );
    
    const RuleAtom ra11 ( &s_real64, 0 );

    const Rule r11 ( &ra11, &r12 );

    const RuleAtom ra10 ( &s_real32, 0 );

    const Rule r10 ( &ra10, &r11 );

    const RuleAtom ra9 ( &s_nat64, 0 );

    const Rule r9 ( &ra9, &r10 );

    const RuleAtom ra8 ( &s_nat32, 0 );

    const Rule r8 ( &ra8, &r9 );

    const RuleAtom ra7 ( &s_nat16, 0 );

    const Rule r7 ( &ra7, &r8 );

    const RuleAtom ra6 ( &s_int64, 0 );

    const Rule r6 ( &ra6, &r7 );

    const RuleAtom ra5 ( &s_int32, 0 );

    const Rule r5 ( &ra5, &r6 );

    const RuleAtom ra4 ( &s_int16, 0 );

    const Rule r4 ( &ra4, &r5 );

    const RuleAtom ra3 ( &s_nat8, 0 );

    const Rule r3 ( &ra3, &r4 );

    const RuleAtom ra2 ( &s_int8, 0 );

    const Rule r2 ( &ra2, &r3 );

    const RuleAtom ra1 ( &s_log8, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_DataType );
}

namespace ReturnType
{
    const RuleAtom ra2 ( &DataType::symbol, 0 );

    const Rule r2 ( &ra2 );

    const RuleAtom ra1 ( &s_void, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_ReturnType );
}

namespace AssignOperator
{
    const RuleAtom ra9 ( &s_vertical_equal, 0 );

    const Rule r9 ( &ra9, 0 );

    const RuleAtom ra8 ( &s_lid_equal, 0 );

    const Rule r8 ( &ra8, &r9 );

    const RuleAtom ra7 ( &s_amper_equal, 0 );

    const Rule r7 ( &ra7, &r8 );

    const RuleAtom ra6 ( &s_plus_equal, 0 );

    const Rule r6 ( &ra6, &r7 );

    const RuleAtom ra5 ( &s_minus_equal, 0 );

    const Rule r5 ( &ra5, &r6 );

    const RuleAtom ra4 ( &s_percent_equal, 0 );

    const Rule r4 ( &ra4, &r5 );

    const RuleAtom ra3 ( &s_slash_equal, 0 );

    const Rule r3 ( &ra3, &r4 );

    const RuleAtom ra2 ( &s_asterisk_equal, 0 );

    const Rule r2 ( &ra2, &r3 );

    const RuleAtom ra1 ( &s_equal, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_AssignOperator );
}

namespace Operation
{

    const RuleAtom ra10 ( &AssignOperator::symbol, 0 );

    const Rule r10 ( &ra10 );

    const RuleAtom ra9 ( &s_less_equal, 0 );

    const Rule r9 ( &ra9, &r10 );

    const RuleAtom ra8 ( &s_greater_equal, 0 );

    const Rule r8 ( &ra8, &r9 );

    const RuleAtom ra7 ( &s_less, 0 );

    const Rule r7 ( &ra7, &r8 );

    const RuleAtom ra6 ( &s_greater, 0 );

    const Rule r6 ( &ra6, &r7 );

    const RuleAtom ra5 ( &s_percent, 0 );

    const Rule r5 ( &ra5, &r6 );

    const RuleAtom ra4 ( &s_slash, 0 );

    const Rule r4 ( &ra4, &r5 );

    const RuleAtom ra3 ( &s_asterisk, 0 );

    const Rule r3 ( &ra3, &r4 );

    const RuleAtom ra2 ( &s_minus, 0 );

    const Rule r2 ( &ra2, &r3 );

    const RuleAtom ra1 ( &s_plus, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_Operation );
}

namespace Number
{
    const RuleAtom ra5 ( &s_exp, 0 );

    const Rule r5 ( &ra5 );

    const RuleAtom ra4 ( &s_real, 0 );

    const Rule r4 ( &ra4, &r5 );

    const RuleAtom ra3 ( &s_hex, 0 );

    const Rule r3 ( &ra3, &r4 );

    const RuleAtom ra2 ( &s_bin, 0 );

    const Rule r2 ( &ra2, &r3 );

    const RuleAtom ra1 ( &s_int, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_Number );
}

/***************************************************************/
//
//                        
//
/***************************************************************/

namespace ExpressionList
{
    extern const GSymbol symbol;
}

namespace Expression2
{
    const RuleAtom ra1_2 ( &ExpressionList::symbol );
    const RuleAtom ra1_1 ( &s_parenthesis_open, &ra1_2 );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_Expression2 );
}

namespace ExpressionParenthesisClose1
{
    extern const GSymbol symbol;
}

namespace Expression1
{
    extern const GSymbol symbol;

    const RuleAtom ra11_2 ( &ExpressionParenthesisClose1::symbol );
    const RuleAtom ra11_1 ( &s_parenthesis_open, &ra11_2 );

    const Rule r11 ( &ra11_1 );

    const RuleAtom ra10_2 ( &symbol );
    const RuleAtom ra10_1 ( &s_minus_minus, &ra10_2 );

    const Rule r10 ( &ra10_1, &r11 );

    const RuleAtom ra9_2 ( &symbol );
    const RuleAtom ra9_1 ( &s_plus_plus, &ra9_2 );

    const Rule r9 ( &ra9_1, &r10 );

    const RuleAtom ra8_2 ( &symbol );
    const RuleAtom ra8_1 ( &s_exclamation, &ra8_2 );

    const Rule r8 ( &ra8_1, &r9 );

    const RuleAtom ra7_2 ( &symbol );
    const RuleAtom ra7_1 ( &s_minus, &ra7_2 );

    const Rule r7 ( &ra7_1, &r8 );

    const RuleAtom ra6_2 ( &symbol );
    const RuleAtom ra6_1 ( &s_plus, &ra6_2 );

    const Rule r6 ( &ra6_1, &r7 );

    const RuleAtom ra5 ( &DataType::symbol );

    const Rule r5 ( &ra5, &r6 );

    const RuleAtom ra4 ( &Number::symbol );

    const Rule r4 ( &ra4, &r5 );

    const RuleAtom ra3 ( &s_name );

    const Rule r3 ( &ra3, &r4 );

    const RuleAtom ra2 ( &s_false );

    const Rule r2 ( &ra2, &r3 );

    const RuleAtom ra1 ( &s_true );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_Expression1 );
}

namespace ExpressionParenthesisClose2
{
    extern const GSymbol symbol;

    const RuleAtom ra3_2 ( &ExpressionParenthesisClose1::symbol );
    const RuleAtom ra3_1 ( &Operation::symbol, &ra3_2 );

    const Rule r3 ( &ra3_1 );

    const RuleAtom ra2_2 ( &ExpressionParenthesisClose2::symbol );
    const RuleAtom ra2_1 ( &Expression2::symbol, & ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_1 ( &s_parenthesis_close );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionParenthesisClose2 );
}

namespace ExpressionParenthesisClose1
{
    const RuleAtom ra1_2 ( &ExpressionParenthesisClose2::symbol );
    const RuleAtom ra1_1 ( &Expression1::symbol, & ra1_2 );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_ExpressionParenthesisClose1 );
}

namespace ExpressionParenthesisClose
{
    const RuleAtom ra2_1 ( &ExpressionParenthesisClose1::symbol );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_1 ( &s_parenthesis_close );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionParenthesisClose );
}

namespace ExpressionBracesClose1
{
    extern const GSymbol symbol;
}

namespace ExpressionBracesClose2
{
    extern const GSymbol symbol;

    const RuleAtom ra3_2 ( &ExpressionBracesClose1::symbol );
    const RuleAtom ra3_1 ( &Operation::symbol, &ra3_2 );

    const Rule r3 ( &ra3_1 );

    const RuleAtom ra2_2 ( &ExpressionBracesClose2::symbol );
    const RuleAtom ra2_1 ( &Expression2::symbol, & ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_1 ( &s_braces_close );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionBracesClose2 );
}

namespace ExpressionBracesClose1
{
    const RuleAtom ra1_2 ( &ExpressionBracesClose2::symbol );
    const RuleAtom ra1_1 ( &Expression1::symbol, & ra1_2 );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_ExpressionBracesClose1 );
}

namespace ExpressionBracesClose
{
    const RuleAtom ra2_1 ( &ExpressionBracesClose1::symbol );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_1 ( &s_braces_close, 0 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionBracesClose );
}

namespace ExpressionSemicolon1
{
    extern const GSymbol symbol;
}

namespace ExpressionSemicolon2
{
    extern const GSymbol symbol;

    const RuleAtom ra3_2 ( &ExpressionSemicolon1::symbol, 0 );
    const RuleAtom ra3_1 ( &Operation::symbol, &ra3_2 );

    const Rule r3 ( &ra3_1 );

    const RuleAtom ra2_2 ( &ExpressionSemicolon2::symbol, 0 );
    const RuleAtom ra2_1 ( &Expression2::symbol, & ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_1 ( &s_semicolon, 0 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionSemicolon2 );
}

namespace ExpressionSemicolon1
{
    const RuleAtom ra1_2 ( &ExpressionSemicolon2::symbol );
    const RuleAtom ra1_1 ( &Expression1::symbol, &ra1_2 );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_ExpressionSemicolon1 );
}

namespace ExpressionSemicolon
{
    const RuleAtom ra2_1 ( &ExpressionSemicolon1::symbol );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_1 ( &s_semicolon, 0 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionSemicolon );
}

namespace ExpressionList3
{
    const RuleAtom ra2_2 ( &ExpressionList::symbol );
    const RuleAtom ra2_1 ( &s_comma, &ra2_2 );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_1 ( &s_parenthesis_close );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionList3 );
}

namespace ExpressionList1
{
    extern const GSymbol symbol;
}

namespace ExpressionList2
{
    extern const GSymbol symbol;

    const RuleAtom ra3_2 ( &ExpressionList1::symbol );
    const RuleAtom ra3_1 ( &Operation::symbol, &ra3_2 );

    const Rule r3 ( &ra3_1 );

    const RuleAtom ra2_2 ( &ExpressionList2::symbol );
    const RuleAtom ra2_1 ( &Expression2::symbol, &ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_1 ( &ExpressionList3::symbol );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionList2 );
}

namespace ExpressionList1
{
    const RuleAtom ra1_2 ( &ExpressionList2::symbol );
    const RuleAtom ra1_1 ( &Expression1::symbol, &ra1_2 );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_ExpressionList1 );
}

namespace ExpressionList
{
    const RuleAtom ra2_1 ( &ExpressionList1::symbol );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_1 ( &s_parenthesis_close );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_ExpressionList );
}

/***************************************************************/
//
//                        
//
/***************************************************************/

namespace StatementList
{
    extern const GSymbol symbol;
}

namespace Statement1
{
    extern const GSymbol symbol;

    const RuleAtom ra7_6 ( &Statement1::symbol, 0 );
    const RuleAtom ra7_5 ( &s_else, &ra7_6 );
    const RuleAtom ra7_4 ( &Statement1::symbol, &ra7_5 );
    const RuleAtom ra7_3 ( &ExpressionParenthesisClose1::symbol, &ra7_4 );
    const RuleAtom ra7_2 ( &s_parenthesis_open, &ra7_3 );
    const RuleAtom ra7_1 ( &s_if, &ra7_2 );

    const Rule r7 ( &ra7_1, 0 );

    const RuleAtom ra6_1 ( &s_break, 0 );

    const Rule r6 ( &ra6_1, &r7 );

    const RuleAtom ra5_1 ( &s_continue, 0 );

    const Rule r5 ( &ra5_1, &r6 );

    const RuleAtom ra4_6 ( &symbol, 0 );
    const RuleAtom ra4_5 ( &ExpressionParenthesisClose::symbol, &ra4_6 );
    const RuleAtom ra4_4 ( &ExpressionSemicolon::symbol, &ra4_5 );
    const RuleAtom ra4_3 ( &ExpressionSemicolon::symbol, &ra4_4 );
    const RuleAtom ra4_2 ( &s_parenthesis_open, &ra4_3 );
    const RuleAtom ra4_1 ( &s_for, &ra4_2 );

    const Rule r4 ( &ra4_1, &r5 );

    const RuleAtom ra3_2 ( &ExpressionSemicolon::symbol, 0 );
    const RuleAtom ra3_1 ( &s_return, &ra3_2 );

    const Rule r3 ( &ra3_1, &r4 );

    const RuleAtom ra2_2 ( &StatementList::symbol, 0 );
    const RuleAtom ra2_1 ( &s_braces_open, &ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_1 ( &ExpressionSemicolon1::symbol, 0 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_Statement1 );
}

namespace DataDefinition2
{
    const RuleAtom ra2_3 ( &ExpressionBracesClose::symbol, 0 );
    const RuleAtom ra2_2 ( &s_braces_open, &ra2_3 );
    const RuleAtom ra2_1 ( &DataType::symbol, &ra2_2 );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_4 ( &ExpressionBracesClose::symbol, 0 );
    const RuleAtom ra1_3 ( &s_braces_open, &ra1_4 );
    const RuleAtom ra1_2 ( &DataType::symbol, &ra1_3 );
    const RuleAtom ra1_1 ( &s_const, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_DataDefinition2 );
}

namespace DataDefinition
{
    const RuleAtom ra2 ( &DataDefinition2::symbol, 0 );
    const RuleAtom ra1 ( &s_name, &ra2 );

    const Rule r1 ( &ra1 );

    const GSymbol symbol ( &r1, tt_DataDefinition );
}

namespace Statement
{
    const RuleAtom ra3_1 ( &Statement1::symbol, 0 );

    const Rule r3 ( &ra3_1 );
    
    const RuleAtom ra2_2 ( &DataDefinition::symbol, 0 );
    const RuleAtom ra2_1 ( &s_static, &ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_2 ( &DataDefinition::symbol, 0 );
    const RuleAtom ra1_1 ( &s_auto, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_Statement );
}

namespace StatementList
{
    extern const GSymbol symbol;

    const RuleAtom ra2_2 ( &StatementList::symbol, 0 );
    const RuleAtom ra2_1 ( &Statement::symbol, &ra2_2 );

    const Rule r2 ( &ra2_1, 0 );

    const RuleAtom ra1 ( &s_braces_close, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_StatementList );
}

/***************************************************************/
//
//                        
//
/***************************************************************/

namespace Argument2
{
    const RuleAtom ra2_2 ( &DataType::symbol, 0 );
    const RuleAtom ra2_1 ( &s_const, &ra2_2 );

    const Rule r2 ( &ra2_1, 0 );

    const RuleAtom ra1 ( &DataType::symbol, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_Argument2 );
}

namespace Argument
{
    const RuleAtom ra2_2 ( &Argument2::symbol, 0 );
    const RuleAtom ra2_1 ( &s_name, &ra2_2 );

    const Rule r2 ( &ra2_1, 0 );

    const RuleAtom ra1 ( &Argument2::symbol, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_Argument );
}

namespace ArgumentList
{
    extern const GSymbol symbol;
}

namespace ArgumentList2
{
    const RuleAtom ra2_2 ( &ArgumentList::symbol, 0 );
    const RuleAtom ra2_1 ( &s_comma, &ra2_2 );

    const Rule r2 ( &ra2_1, 0 );

    const RuleAtom ra1 ( &s_parenthesis_close, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_ArgumentList2 );
}

namespace ArgumentList
{
    const RuleAtom ra2_2 ( &ArgumentList2::symbol, 0 );
    const RuleAtom ra2_1 ( &Argument::symbol, &ra2_2 );

    const Rule r2 ( &ra2_1, 0 );

    const RuleAtom ra1 ( &s_parenthesis_close, 0 );

    const Rule r1 ( &ra1, &r2 );

    const GSymbol symbol ( &r1, tt_ArgumentList );
}

namespace SubProgramType
{
    const RuleAtom ra3 ( &ReturnType::symbol, 0 );
    const RuleAtom ra2 ( &ArgumentList::symbol, &ra3 );
    const RuleAtom ra1 ( &s_parenthesis_open, &ra2 );

    const Rule r1 ( &ra1 );

    const GSymbol symbol ( &r1, tt_SubProgramType );
}

namespace Type
{
    const RuleAtom ra3_1 ( &SubProgramType::symbol, 0 );

    const Rule r3 ( &ra3_1, 0 );

    const RuleAtom ra2_1 ( &DataType::symbol, 0 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_2 ( &DataType::symbol, 0 );
    const RuleAtom ra1_1 ( &s_const, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_Type );
}

namespace Definition2
{
    const RuleAtom ra1_3 ( &ExpressionBracesClose::symbol );
    const RuleAtom ra1_2 ( &s_braces_open, &ra1_3 );
    const RuleAtom ra1_1 ( &DataType::symbol, &ra1_2 );

    const Rule r1 ( &ra1_1, 0 );

    const GSymbol symbol ( &r1, tt_Definition2 );
}

namespace Definition
{
    const RuleAtom ra3_3 ( &StatementList::symbol );
    const RuleAtom ra3_2 ( &s_braces_open, &ra3_3 );
    const RuleAtom ra3_1 ( &SubProgramType::symbol, &ra3_2 );

    const Rule r3 ( &ra3_1, 0 );

    const RuleAtom ra2_1 ( &Definition2::symbol );

    const Rule r2 ( &ra2_1, &r3);

    const RuleAtom ra1_2 ( &Definition2::symbol );
    const RuleAtom ra1_1 ( &s_const, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_Definition );
}

namespace Declaration
{
    const RuleAtom ra3_3 ( &Type::symbol );
    const RuleAtom ra3_2 ( &s_name, &ra3_3 );
    const RuleAtom ra3_1 ( &s_extern, &ra3_2 );

    const Rule r3 ( &ra3_1 );

    const RuleAtom ra2_3 ( &Definition::symbol );
    const RuleAtom ra2_2 ( &s_name, &ra2_3 );
    const RuleAtom ra2_1 ( &s_global, &ra2_2 );

    const Rule r2 ( &ra2_1, &r3 );

    const RuleAtom ra1_3 ( &Definition::symbol );
    const RuleAtom ra1_2 ( &s_name, &ra1_3 );
    const RuleAtom ra1_1 ( &s_local, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_Declaration );
}

namespace DeclarationList
{
    extern const GSymbol symbol;

    const RuleAtom ra2_1 ( &s_eof );

    const Rule r2 ( &ra2_1 );

    const RuleAtom ra1_2 ( &symbol );
    const RuleAtom ra1_1 ( &Declaration::symbol, &ra1_2 );

    const Rule r1 ( &ra1_1, &r2 );

    const GSymbol symbol ( &r1, tt_DeclarationList );
}

namespace Program
{
    const RuleAtom ra1_1 ( &DeclarationList::symbol );

    const Rule r1 ( &ra1_1 );

    const GSymbol symbol ( &r1, tt_Program );
}

const RuleAtom * const start = & Program::ra1_1;
