
#include "Translate.h"
#include "syntax.h"
#include "semantic.h"

//  
LexReturn lex_analyse ( nat32 size, const char8 * source, Suite<Token> & result );

//     
void keywords ( Suite<Token> & result );

bool operator == ( const Token & t1, const Token & t2 )
{
    if ( t1.size != t2.size ) return false;
    nat32 i = 0;
    for ( ; i < t1.size; ++i ) if ( t1.text[i] != t2.text[i] ) return false;
    return true;
}

void operator += ( Suite<char8> & suite, const char * str )
{
    if ( str == 0 ) return;
    nat32 i = 0;
    while ( str[i] != 0 ) suite.addAftLas ( str[i++] );
}

inline
void operator += ( Suite<char8> & suite, const Token & token )
{
    suite.addAftLas ( CArrRef<char> ( token.text, token.size ) );
}

const char8 * string ( TokenType type )
{
    switch ( type )
    {
    case tt_exclamation      : return "!";   // !   33
    case tt_sharp            : return "#";   // #  35
    case tt_percent          : return "%";   // %  37
    case tt_amper            : return "&";   // & 
    case tt_parenthesis_open : return "(";   // (   
    case tt_parenthesis_close: return ")";   // )   
    case tt_asterisk         : return "*";   // *  42
    case tt_plus             : return "+";   // + 
    case tt_comma            : return ",";   // , 
    case tt_minus            : return "-";   // - 
    case tt_dot              : return ".";   // . 
    case tt_slash            : return "/";   // /  
    case tt_colon            : return ":";   // : 
    case tt_semicolon        : return ";";   // ;   
    case tt_less             : return "<";   // < 
    case tt_equal            : return "=";   // = 
    case tt_greater          : return ">";   // >  62
    case tt_question         : return "?";   // ?  63
    case tt_at               : return "@";   // @   64
    case tt_bracket_open     : return "[";   // [    91
    case tt_bracket_close    : return "]";   // ]    93
    case tt_lid              : return "^";   // ^  94
    case tt_braces_open      : return "{";   // {    123
    case tt_vertical         : return "|";   // |  
    case tt_braces_close     : return "}";   // }   
    case tt_tilde            : return "~";   // ~  126
//  
    case tt_exclamation_equal    : return "!="; //  !=
    case tt_percent_equal        : return "%="; //  %=
    case tt_amper_equal          : return "&="; //  &=
    case tt_asterisk_equal       : return "*="; //  *=
    case tt_plus_plus            : return "++"; //  ++
    case tt_plus_equal           : return "+="; //  +=
    case tt_minus_minus          : return "--"; //  --
    case tt_minus_equal          : return "-="; //  -=
    case tt_slash_equal          : return "/="; //  /=
    case tt_less_less            : return "<<"; //  <<
    case tt_less_less_equal      : return "<<=";// <<=
    case tt_less_equal           : return "<="; //  <=
    case tt_equal_equal          : return "=="; //  ==
    case tt_greater_greater      : return ">>"; //  >>
    case tt_greater_equal        : return ">="; //  >=
    case tt_greater_greater_equal: return ">>=";// >>=
    case tt_lid_equal            : return "^="; //  ^=
    case tt_vertical_equal       : return "|="; //  |=
    }
    return 0;
}

const Token t_Program = { tt_Program, 0, 0 };
const Token t_DeclarationList = { tt_DeclarationList, 0, 0 };

inline void addRight ( Suite<TreeNode> & tree, nat32 i1, nat32 i2 )
{
    tree[i1].right = i2;
    tree[i2].up = tree[i1].up;
    tree[i2].left = i1;
    tree[i2].down =
    tree[i2].right = 0;
}

static
bool syntax ( CArrRef<Token> source, Suite<TreeNode> & tree, Suite<char8> & result )
{
    tree[0].token = t_Program;
    tree[0].up = tree[0].left = tree[0].right = 0;
    tree[0].down = 1;
    tree[1].token = t_DeclarationList;
    tree[1].up = tree[1].down = tree[1].left = tree[1].right = 0;
    ListRAP list;
    list.addAftCur ( new ItemRAP ( start ) );
    list.cur()->info = 1;
    bool ok = true;
    nat32 i;
    for ( i = 0; i < source.size(); ++i )
    {
        const Token & token = source[i];
        result += token;
        if ( token.type == tt_comment )
        {
            continue;
        }
        const GSymbol * symbol = list.cur()->d->symbol;
        if ( symbol->rule == 0 )
        {
            if ( symbol->type == token.type )
            {
                tree[list.cur()->info].token = token;
                if ( i > 0 && source[i-1].type == tt_comment )
                {
                    tree[list.cur()->info].comment = & source[i-1];
                }
            }
            else
            {
                ok = false;
            }
        }
        else
        while ( symbol->rule != 0 )
        {
            const RuleAtom * a = 0;
            Show<ItemRAP> show ( symbol->list );
            if ( show.top() )
            do
            {
                if ( show.cur()->info == token.type )
                {
                    a = show.cur()->d;
                    symbol = a->symbol;
                    list.cur()->d = a;
                    TreeNode node;
                    node.comment = 0;
                    node.up = list.cur()->info;
                    node.down = node.left = node.right = 0;
                    if ( a->symbol->rule == 0 )
                    {
                        node.token = token;
                        tree[node.up].down = list.cur()->info = tree.addAftLas ( node );
                        if ( i > 0 && source[i-1].type == tt_comment )
                        {
                            tree[list.cur()->info].comment = & source[i-1];
                        }
                    }
                    else
                    {
                        node.token.type = symbol->type;
                        node.token.text = 0;
                        node.token.size = 0;
                        tree[node.up].down = list.cur()->info = tree.addAftLas ( node );
                    }
                    nat32 pre = list.cur()->info;
                    for(;;)
                    {
                        a = a->next;
                        if ( a == 0 ) break;
                        list.addBefCur ( new ItemRAP ( a ) );
                        node.token.type = a->symbol->type;
                        node.token.text = 0;
                        node.token.size = 0;
                        node.comment = 0;
                        const nat32 ix = list.cur()->info = tree.addAftLas ( node );
                        addRight ( tree, pre, ix );
                        pre = ix;
                    }
                    list.end();
                    goto m1;
                }
            }
            while ( show.next() );
            ok = false;
            break;
m1:;    }
        if ( ok )
        {
            list.delCur();
        }
        else
        {
            const char8 * str = string ( symbol->type );
            if ( str == 0 )
            {
                result += "!! :  ";
            }
            else
            {
                result += "!! :    ";
                result += str;
            }
            break;
        }
    }
    return ok;
}

static
void dummy ( )
{
    nat32 i = 0;
}

static
void testTree ( const TreeNode * tree )
{
    nat32 i = 0;
    for(;;)
    {
m1:     const TreeNode & node = tree[i];
        if ( node.down != 0 )
        {
            if ( tree[node.down].up != i ) dummy();
            i = node.down;
            if ( tree[i].left != 0 ) dummy();
            continue;
        }
        if ( node.right != 0 )
        {
            if ( tree[node.right].left != i ) dummy();
            i = node.right;
            if ( tree[i].up != node.up ) dummy();
            continue;
        }
        for(;;)
        {
            i = tree[i].up;
            if ( i == 0 ) return;
            if ( tree[i].right != 0 )
            {
                if ( tree[tree[i].right].left != i ) dummy();
                i = tree[i].right;
                if ( tree[i].up != tree[tree[i].left].up ) dummy();
                goto m1;
            }
        }
        break;
    }
}

static
void output ( nat32 i, const Token * vtoken, const TreeNode * tree, Suite<char8> & result )
{
    const nat32 ii = i;
    for(;;)
    {
m1:     const TreeNode & node = tree[i];
        if ( node.down != 0 )
        {
            i = node.down;
            continue;
        }
        if ( node.comment != 0 ) result += * node.comment;
        result += node.token;
        if ( node.right != 0 )
        {
            i = node.right;
            continue;
        }
        for(;;)
        {
            i = tree[i].up;
            if ( i <= ii ) return;
            if ( tree[i].right != 0 )
            {
                i = tree[i].right;
                goto m1;
            }
        }
        break;
    }
}

bool translate ( nat32 size, const char8 * source, Suite<char8> & result ) 
{
    Suite<Token> vtoken; //   
//  
    LexReturn lr = lex_analyse ( size, source, vtoken );
//    ,         
    if ( lr != lr_ok )
    {
        result += cfor012 ( vtoken );
        switch ( lr )
        {
        case lr_bad_bin:
            {
                result += "!!   ";
                return false;
            }
        case lr_bad_hex:
            {
                result += "!!   ";
                return false;
            }
        case lr_bad_num:
            {
                result += "!!   ";
                return false;
            }
        case lr_no_quote1:
            {
                result += "!! '    ";
                return false;
            }
        case lr_no_quote2:
            {
                result += "!! \"    ";
                return false;
            }
        case lr_open_comment:
            {
                result += "!! /*   ";
                return false;
            }
        case lr_unknown_character:
            {
                result += "!! c c";
                return false;
            }
        }
    }
//     
    keywords ( vtoken );
//  
    Token eof;
    eof.type = tt_eof;
    eof.text = 0;
    eof.size = 0;
    vtoken.addAftLas ( eof );
    Suite<TreeNode> tree ( 2 ); //  
    if ( syntax ( vtoken, tree, result ) )
    {
//  
        testTree ( &tree[0] );
        semantic ( tree );
        testTree ( &tree[0] );
        result.resize ( 0 );
        output ( 0, &vtoken[0], &tree[0], result );
        return true;
    }
    return false;
}
